from galaxy_ng.app import models
from guardian.shortcuts import get_objects_for_user

from .namespace import NamespaceViewSet


class MyNamespaceViewSet(NamespaceViewSet):
    def get_queryset(self):
        return get_objects_for_user(
            self.request.user,
            ('galaxy.change_namespace', 'galaxy.upload_to_namespace'),
            any_perm=True,
            klass=models.Namespace
        )



from rest_framework import mixins
from django.http import Http404
from pulp_ansible.app.models import AnsibleDistribution
from galaxy_ng.app.access_control import access_policy
from galaxy_ng.app.api import base as api_base
from ..serializers.sync import CollectionRemoteSerializer


class SyncConfigViewSet(
    mixins.RetrieveModelMixin,
    mixins.UpdateModelMixin,
    api_base.GenericViewSet,
):
    serializer_class = CollectionRemoteSerializer
    permission_classes = [access_policy.CollectionRemoteAccessPolicy]

    def get_object(self):
        distribution = AnsibleDistribution.objects.get(base_path=self.kwargs['path'])
        if distribution and distribution.repository:
            return distribution.repository.remote.ansible_collectionremote
        raise Http404



from django.db import models
from pulpcore.plugin import models as pulp_models
from pulp_container.app import models as container_models
from django_lifecycle import LifecycleModelMixin

from galaxy_ng.app.access_control import mixins


class ContainerDistribution(
        container_models.ContainerDistribution,
        LifecycleModelMixin,
        mixins.GroupModelPermissionsMixin):

    class Meta:
        proxy = True
        default_related_name = "%(app_label)s_%(model_name)s"


class ContainerNamespace(
        container_models.ContainerNamespace,
        LifecycleModelMixin,
        mixins.GroupModelPermissionsMixin):

    class Meta:
        proxy = True
        default_related_name = "%(app_label)s_%(model_name)s"


class ContainerDistroReadme(models.Model):
    container = models.OneToOneField(
        container_models.ContainerDistribution,
        on_delete=models.CASCADE, primary_key=True)
    created = models.DateTimeField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True, null=True)
    text = models.TextField(blank=True)


class ContainerRegistryRemote(
    pulp_models.Remote,
    mixins.GroupModelPermissionsMixin,
):

    class Meta:
        default_related_name = "%(app_label)s_%(model_name)s"


class ContainerRegistryRepos(models.Model):
    registry = models.ForeignKey(
        ContainerRegistryRemote,
        on_delete=models.CASCADE,
    )
    repository_remote = models.OneToOneField(
        container_models.ContainerRemote,
        on_delete=models.CASCADE,
        primary_key=True,
    )


class ContainerSyncTask(models.Model):
    repository = models.ForeignKey(
        container_models.ContainerRepository,
        on_delete=models.CASCADE,
    )
    task = models.ForeignKey(
        pulp_models.Task,
        on_delete=models.CASCADE,
    )

    class Meta:
        ordering = ['-task__finished_at']



from django.db import transaction
from rest_framework.authentication import BasicAuthentication
from rest_framework.request import Request
from rest_framework.authtoken.models import Token
from rest_framework.response import Response
from rest_framework import status as http_code

from galaxy_ng.app.api import base as api_base
from galaxy_ng.app.access_control import access_policy


class TokenView(api_base.APIView):
    authentication_classes = (BasicAuthentication, *api_base.GALAXY_AUTHENTICATION_CLASSES)
    permission_classes = [access_policy.TokenAccessPolicy]

    @transaction.atomic
    def post(self, request: Request, *args, **kwargs) -> Response:
        """Create or refresh user token."""
        Token.objects.filter(user=self.request.user).delete()
        token = Token.objects.create(user=self.request.user)
        return Response({"token": token.key})

    def delete(self, request, *args, **kwargs):
        """Invalidate user token."""
        Token.objects.filter(user=self.request.user).delete()
        return Response(status=http_code.HTTP_204_NO_CONTENT)



from django.db import models
from django.db.models.signals import post_save
from django.dispatch import receiver
from pulpcore.plugin.models import Task
from pulp_ansible.app.models import AnsibleRepository, Collection

from .namespace import Namespace


@receiver(post_save, sender=Collection)
def create_namespace_if_not_present(sender, instance, created, **kwargs):
    """Ensure Namespace object exists when Collection object saved.

    django signal for pulp_ansible Collection model, so that whenever a
    Collection object is created or saved, it will create a Namespace object
    if the Namespace does not already exist.

    Supports use case: In pulp_ansible sync, when a new collection is sync'd
    a new Collection object is created, but the Namespace object is defined
    in galaxy_ng and therefore not created. This signal ensures the
    Namespace is created.
    """

    Namespace.objects.get_or_create(name=instance.namespace)


class CollectionSyncTask(models.Model):
    repository = models.ForeignKey(AnsibleRepository, on_delete=models.CASCADE)
    task = models.ForeignKey(Task, on_delete=models.CASCADE)

    class Meta:
        ordering = ['-task__pulp_last_updated']



from rest_framework import serializers

from pulp_ansible.app.models import CollectionImport as PulpCollectionImport


class ImportTaskListSerializer(serializers.ModelSerializer):
    """
    A serializer for a CollectionImport list view.
    """

    id = serializers.UUIDField(source="pk")
    state = serializers.CharField(source="task.state")
    namespace = serializers.CharField(source="galaxy_import.namespace.name")
    name = serializers.CharField(source="galaxy_import.name")
    version = serializers.CharField(source="galaxy_import.version")
    created_at = serializers.DateTimeField(source="task.pulp_created")
    updated_at = serializers.DateTimeField(source="task.pulp_last_updated")
    started_at = serializers.DateTimeField(source="task.started_at")
    finished_at = serializers.DateTimeField(source="task.finished_at")

    class Meta:
        model = PulpCollectionImport
        fields = ("id", "state", "namespace", "name", "version",
                  "created_at", "updated_at", "started_at", "finished_at")


class ImportTaskDetailSerializer(ImportTaskListSerializer):

    error = serializers.JSONField(source="task.error")
    messages = serializers.JSONField()

    class Meta(ImportTaskListSerializer.Meta):
        fields = ImportTaskListSerializer.Meta.fields + \
            ('error', 'messages')



from django.db import transaction
from django.db.models import Q
from django_filters import filters
from django_filters.rest_framework import filterset, DjangoFilterBackend

from galaxy_ng.app import models
from galaxy_ng.app.access_control.access_policy import NamespaceAccessPolicy
from galaxy_ng.app.api import base as api_base
from galaxy_ng.app.api.v3 import serializers
from galaxy_ng.app.exceptions import ConflictError


class NamespaceFilter(filterset.FilterSet):
    keywords = filters.CharFilter(method='keywords_filter')

    sort = filters.OrderingFilter(
        fields=(
            ('name', 'name'),
            ('company', 'company'),
            ('id', 'id'),
        ),
    )

    class Meta:
        model = models.Namespace
        fields = ('name', 'company',)

    def keywords_filter(self, queryset, name, value):

        keywords = self.request.query_params.getlist('keywords')

        for keyword in keywords:
            queryset = queryset.filter(Q(name__icontains=keyword) | Q(company__icontains=keyword))

        return queryset


class NamespaceViewSet(api_base.ModelViewSet):
    lookup_field = "name"
    queryset = models.Namespace.objects.all()
    serializer_class = serializers.NamespaceSerializer
    filter_backends = (DjangoFilterBackend,)
    filterset_class = NamespaceFilter
    swagger_schema = None
    permission_classes = [NamespaceAccessPolicy]

    def get_serializer_class(self):
        if self.action == 'list':
            return serializers.NamespaceSummarySerializer
        else:
            return self.serializer_class

    @transaction.atomic
    def create(self, request, *args, **kwargs):
        """Override to validate for name duplication before serializer validation."""
        name = request.data.get('name')
        if name and models.Namespace.objects.filter(name=name).exists():
            # Ensures error raised is 409, not 400.
            raise ConflictError(
                detail={'name': f'A namespace named {name} already exists.'}
            )
        return super().create(request, *args, **kwargs)



from django.db import migrations

def create_inbound_repo_per_namespace(apps, schema_editor):
    AnsibleRepository = apps.get_model('ansible', 'AnsibleRepository')
    AnsibleDistribution = apps.get_model('ansible', 'AnsibleDistribution')
    Namespace = apps.get_model('galaxy', 'Namespace')
    db_alias = schema_editor.connection.alias

    for namespace in Namespace.objects.using(db_alias).all():
        name = f'inbound-{namespace.name}'
        repo = AnsibleRepository.objects.using(db_alias).create(
            name=name,
            pulp_type='ansible.ansible',
        )
        distro = AnsibleDistribution.objects.using(db_alias).create(
            name=name,
            base_path=name,
            repository=repo,
            pulp_type='ansible.ansible',
        )


class Migration(migrations.Migration):

    dependencies = [
        ('galaxy', '0002_add_synclist_20200330_squashed'),
    ]

    operations = [
        migrations.RunPython(create_inbound_repo_per_namespace, elidable=True)
    ]



from .collection import (
    CollectionSerializer,
    CollectionVersionSerializer,
    CollectionVersionListSerializer,
    CollectionUploadSerializer,
    UnpaginatedCollectionVersionSerializer,
)

from .namespace import (
    NamespaceSerializer,
    NamespaceSummarySerializer,
)

from .group import (
    GroupSummarySerializer,
)

from .task import (
    TaskSerializer,
    TaskSummarySerializer,
)


__all__ = (
    'CollectionSerializer',
    'CollectionVersionSerializer',
    'CollectionVersionListSerializer',
    'CollectionUploadSerializer',
    'GroupSummarySerializer',
    'NamespaceSerializer',
    'NamespaceSummarySerializer',
    'TaskSerializer',
    'TaskSummarySerializer',
    'UnpaginatedCollectionVersionSerializer',
)



import logging
import unittest

from django.test import override_settings
from django.conf import settings
from rest_framework import status as http_code

from guardian import shortcuts

from galaxy_ng.app.models import auth as auth_models
from galaxy_ng.app.constants import DeploymentMode

from . import base
from . import rh_auth
from .synclist_base import BaseSyncListViewSet, ACCOUNT_SCOPE

log = logging.getLogger(__name__)


class BaseUiSynclistViewSet:
    """Test SyncListViewSet as an admin / pe_group member"""

    def setUp(self):
        super().setUp()
        if settings.GALAXY_DEPLOYMENT_MODE != DeploymentMode.INSIGHTS.value:
            raise unittest.SkipTest("Skipping insights mode tests since we are in standalone mode.")

        self.account_number = "987654"
        self.user = auth_models.User.objects.create(username="admin")
        self.group = self._create_partner_engineer_group()
        self.user.groups.add(self.group)
        self.user.save()

        self.synclist_name = "test_synclist"
        self.synclist = self._create_synclist(
            name=self.synclist_name,
            repository=self.repo,
            upstream_repository=self.default_repo,
            groups=[self.group],
        )

        self.synclist.save()

        self.client.force_authenticate(user=None)

    def _group(self, user_group, perms=None):
        perms = perms or self.default_owner_permissions
        group = {
            "id": user_group.id,
            "name": user_group.name,
            "object_permissions": perms,
        }
        return group

    def test_synclist_create(self):
        new_synclist_name = "new_synclist"
        post_data = {
            "repository": self.repo.pulp_id,
            "collections": [],
            "namespaces": [],
            "policy": "include",
            "name": new_synclist_name,
            "groups": [self._group(self.group)],
        }

        synclists_url = base.get_current_ui_url("synclists-list")

        log.debug("self.synclist: %s", self.synclist)

        response = self.client.post(synclists_url, post_data, format="json")

        log.debug("response: %s", response)
        log.debug("response.data: %s", response.data)

        self.assertEqual(response.status_code, http_code.HTTP_201_CREATED, msg=response.data)

    def test_synclist_update(self):
        ns1_name = "unittestnamespace1"
        ns2_name = "unittestnamespace2"
        ns1 = self._create_namespace(ns1_name, groups=[self.group])
        ns2 = self._create_namespace(ns2_name, groups=[self.group])
        ns1.save()
        ns2.save()

        post_data = {
            "repository": self.repo.pulp_id,
            "collections": [],
            "namespaces": [ns1_name, ns2_name],
            "policy": "include",
            "groups": [self._group(self.group)],
        }

        synclists_detail_url = base.get_current_ui_url(
            "synclists-detail", kwargs={"pk": self.synclist.id}
        )

        response = self.client.patch(synclists_detail_url, post_data, format="json")

        log.debug("response: %s", response)
        log.debug("response.data: %s", response.data)

        self.assertEqual(response.status_code, http_code.HTTP_200_OK, msg=response.data)
        self.assertIn("name", response.data)
        self.assertIn("repository", response.data)
        self.assertEqual(response.data["name"], self.synclist_name)
        self.assertEqual(response.data["policy"], "include")

    def test_synclist_list(self):
        log.debug('GALAXY_DEPLOYMENT_MODE: %s', settings.GALAXY_DEPLOYMENT_MODE)

        synclists_url = base.get_current_ui_url("synclists-list")
        response = self.client.get(synclists_url)

        log.debug("response.data: %s", response.data)

        self.assertEqual(response.status_code, http_code.HTTP_200_OK, msg=response.data)

    def test_synclist_list_empty(self):
        synclists_url = base.get_current_ui_url("synclists-list")

        response = self.client.get(synclists_url)

        log.debug("response: %s", response)
        log.debug("data: %s", response.data)

        self.assertEqual(response.status_code, http_code.HTTP_200_OK, msg=response.data)

    def test_synclist_detail(self):
        synclists_detail_url = base.get_current_ui_url(
            "synclists-detail", kwargs={"pk": self.synclist.id}
        )

        response = self.client.get(synclists_detail_url)

        self.assertEqual(response.status_code, http_code.HTTP_200_OK, msg=response.data)
        self.assertIn("name", response.data)
        self.assertIn("repository", response.data)
        self.assertEqual(response.data["name"], self.synclist_name)
        self.assertEqual(response.data["policy"], "exclude")
        self.assertEqual(response.data["collections"], [])
        self.assertEqual(response.data["namespaces"], [])

    def test_synclist_delete(self):
        synclists_detail_url = base.get_current_ui_url(
            "synclists-detail", kwargs={"pk": self.synclist.id}
        )

        log.debug("delete url: %s", synclists_detail_url)

        response = self.client.delete(synclists_detail_url)

        log.debug("delete response: %s", response)

        self.assertEqual(response.status_code, http_code.HTTP_204_NO_CONTENT, msg=response.data)


@override_settings(
    GALAXY_AUTHENTICATION_CLASSES=["galaxy_ng.app.auth.auth.RHIdentityAuthentication"],
)
class TestUiSynclistViewSetWithGroupPerms(BaseUiSynclistViewSet, BaseSyncListViewSet):
    def setUp(self):
        super().setUp()
        if settings.GALAXY_DEPLOYMENT_MODE != DeploymentMode.INSIGHTS.value:
            raise unittest.SkipTest("Skipping insights mode tests since we are in standalone mode.")

        self.user = auth_models.User.objects.create_user(username="test1", password="test1-secret")
        self.group = self._create_group_with_synclist_perms(
            ACCOUNT_SCOPE, self.account_number, users=[self.user]
        )
        self.user.save()
        self.group.save()

        self.user.groups.add(self.group)
        self.user.save()

        self.synclist_name = "test_synclist"
        self.synclist = self._create_synclist(
            name=self.synclist_name,
            repository=self.repo,
            upstream_repository=self.default_repo,
            groups=[self.group],
        )

        self.credentials()

    def credentials(self):
        x_rh_identity = rh_auth.user_x_rh_identity(self.user.username, self.group.account_number())
        log.debug('x_rh_identity: %s', x_rh_identity)

        self.client.credentials(HTTP_X_RH_IDENTITY=x_rh_identity)
        log.debug('self.client.credentials: %s', self.client.credentials)


class DeniedSynclistViewSet(BaseUiSynclistViewSet):
    def setUp(self):
        super().setUp()

    def test_synclist_create(self):
        post_data = {
            "repository": self.repo.pulp_id,
            "collections": [],
            "namespaces": [],
            "policy": "include",
            "name": "new_synclist",
            "groups": [self._group(self.group)],
        }

        synclists_url = base.get_current_ui_url("synclists-list")
        response = self.client.post(synclists_url, post_data, format="json")

        self.assertEqual(response.status_code, http_code.HTTP_403_FORBIDDEN, msg=response.data)

    def test_synclist_detail(self):
        synclists_detail_url = base.get_current_ui_url(
            "synclists-detail", kwargs={"pk": self.synclist.id}
        )

        response = self.client.get(synclists_detail_url)
        self.assertEqual(response.status_code, http_code.HTTP_403_FORBIDDEN, msg=response.data)

    def test_synclist_list(self):
        synclists_url = base.get_current_ui_url("synclists-list")
        response = self.client.get(synclists_url)
        self.assertEqual(response.status_code, http_code.HTTP_403_FORBIDDEN, msg=response.data)

    def test_synclist_list_empty(self):
        synclists_url = base.get_current_ui_url("synclists-list")
        response = self.client.get(synclists_url)
        self.assertEqual(response.status_code, http_code.HTTP_403_FORBIDDEN, msg=response.data)

    def test_synclist_update(self):
        post_data = {
            "repository": self.repo.pulp_id,
            "collections": [],
            "namespaces": [],
            "policy": "include",
            "groups": [self._group(self.group)],
        }

        synclists_detail_url = base.get_current_ui_url(
            "synclists-detail", kwargs={"pk": self.synclist.id}
        )

        response = self.client.patch(synclists_detail_url, post_data, format="json")

        self.assertEqual(response.status_code, http_code.HTTP_403_FORBIDDEN, msg=response.data)

    def test_synclist_delete(self):
        synclists_detail_url = base.get_current_ui_url(
            "synclists-detail", kwargs={"pk": self.synclist.id}
        )

        response = self.client.delete(synclists_detail_url)
        self.assertEqual(response.status_code, http_code.HTTP_403_FORBIDDEN, msg=response.data)


@override_settings(
    GALAXY_AUTHENTICATION_CLASSES=["galaxy_ng.app.auth.auth.RHIdentityAuthentication"],
)
class TestUiSynclistViewSetWithDefaultGroupPerms(DeniedSynclistViewSet, BaseSyncListViewSet):
    def setUp(self):
        super().setUp()
        if settings.GALAXY_DEPLOYMENT_MODE != DeploymentMode.INSIGHTS.value:
            raise unittest.SkipTest("Skipping insights mode tests since we are in standalone mode.")

        self.user = auth_models.User.objects.create_user(username="test1", password="test1-secret")
        self.group = self._create_group(ACCOUNT_SCOPE, self.account_number, users=[self.user])
        self.user.save()
        self.group.save()

        self.user.groups.add(self.group)
        self.user.save()

        self.synclist_name = "test_synclist"
        self.synclist = self._create_synclist(
            name=self.synclist_name,
            repository=self.repo,
            upstream_repository=self.default_repo,
            groups=[self.group],
        )

        self.credentials()

    def credentials(self):
        x_rh_identity = rh_auth.user_x_rh_identity(self.user.username, self.group.account_number())
        self.client.credentials(HTTP_X_RH_IDENTITY=x_rh_identity)

    def test_synclist_detail(self):
        synclists_detail_url = base.get_current_ui_url(
            "synclists-detail", kwargs={"pk": self.synclist.id}
        )

        response = self.client.get(synclists_detail_url)

        # If we created the synclist with group perms directly, we can GET detail
        self.assertEqual(response.status_code, http_code.HTTP_200_OK, msg=response.data)
        self.assertIn("name", response.data)
        self.assertIn("repository", response.data)
        self.assertEqual(response.data["name"], self.synclist_name)
        self.assertEqual(response.data["policy"], "exclude")
        self.assertEqual(response.data["collections"], [])
        self.assertEqual(response.data["namespaces"], [])

    def test_synclist_update(self):
        ns1_name = "unittestnamespace1"
        ns1 = self._create_namespace(ns1_name, groups=[self.group])
        ns1.save()

        post_data = {
            "repository": self.repo.pulp_id,
            "collections": [],
            "namespaces": [ns1_name],
            "policy": "include",
            "groups": [self._group(self.group)],
        }

        synclists_detail_url = base.get_current_ui_url(
            "synclists-detail", kwargs={"pk": self.synclist.id}
        )

        response = self.client.patch(synclists_detail_url, post_data, format="json")

        log.debug("response: %s", response)
        log.debug("response.data: %s", response.data)

        # If we created the synclist with group perms directly, we can modify it
        self.assertEqual(response.status_code, http_code.HTTP_200_OK, msg=response.data)
        self.assertIn("name", response.data)
        self.assertIn("repository", response.data)
        self.assertEqual(response.data["name"], self.synclist_name)
        self.assertEqual(response.data["policy"], "include")


@override_settings(
    GALAXY_AUTHENTICATION_CLASSES=["galaxy_ng.app.auth.auth.RHIdentityAuthentication"],
)
class TestUiSynclistViewSetNoGroupPerms(DeniedSynclistViewSet, BaseSyncListViewSet):
    def setUp(self):
        super().setUp()
        if settings.GALAXY_DEPLOYMENT_MODE != DeploymentMode.INSIGHTS.value:
            raise unittest.SkipTest("Skipping insights mode tests since we are in standalone mode.")
        self.user = auth_models.User.objects.create_user(
            username="test_user_noperms", password="test1-secret"
        )
        self.group = self._create_group(ACCOUNT_SCOPE, self.account_number, users=[self.user])
        self.user.save()
        self.group.save()

        # Remove any group level perms
        for perm in self.default_owner_permissions:
            shortcuts.remove_perm(f"galaxy.{perm}", self.group)
        self.group.save()

        self.synclist_name = "test_synclist"
        self.synclist = self._create_synclist(
            name=self.synclist_name,
            repository=self.repo,
            upstream_repository=self.default_repo,
            groups=[self.group],
        )
        for perm in self.default_owner_permissions:
            shortcuts.remove_perm(f"galaxy.{perm}", self.group, self.synclist)

        self.credentials()
        self.client.force_authenticate(user=self.user)

    def credentials(self):
        x_rh_identity = rh_auth.user_x_rh_identity(self.user.username, self.group.account_number())
        self.client.credentials(HTTP_X_RH_IDENTITY=x_rh_identity)



from django.conf import settings

from rest_framework import generics
from rest_framework import permissions
from rest_framework import views
from rest_framework import viewsets
from rest_framework.settings import perform_import


GALAXY_EXCEPTION_HANDLER = perform_import(
    settings.GALAXY_EXCEPTION_HANDLER,
    'GALAXY_EXCEPTION_HANDLER'
)
GALAXY_AUTHENTICATION_CLASSES = perform_import(
    settings.GALAXY_AUTHENTICATION_CLASSES,
    'GALAXY_AUTHENTICATION_CLASSES'
)
GALAXY_PAGINATION_CLASS = perform_import(
    settings.GALAXY_PAGINATION_CLASS,
    'GALAXY_PAGINATION_CLASS'
)


class _MustImplementPermission(permissions.BasePermission):
    def has_permission(self, request, view):
        raise NotImplementedError("subclass must implement permission_classes")


class LocalSettingsMixin:
    authentication_classes = GALAXY_AUTHENTICATION_CLASSES
    pagination_class = GALAXY_PAGINATION_CLASS
    permission_classes = [_MustImplementPermission]

    def get_exception_handler(self):
        return GALAXY_EXCEPTION_HANDLER


class APIView(LocalSettingsMixin, views.APIView):
    pass


class ViewSet(LocalSettingsMixin, viewsets.ViewSet):
    pass


class GenericAPIView(LocalSettingsMixin, generics.GenericAPIView):
    pass


class GenericViewSet(LocalSettingsMixin, viewsets.GenericViewSet):
    pass


class ModelViewSet(LocalSettingsMixin, viewsets.ModelViewSet):
    pass



import json

from rest_framework import serializers

from guardian.shortcuts import get_users_with_perms

from pulp_container.app import models as container_models
from pulpcore.plugin import models as core_models

from galaxy_ng.app import models
from galaxy_ng.app.access_control.fields import GroupPermissionField, MyPermissionsField

namespace_fields = (
    'name',
    'my_permissions',
    'owners'
)


class ContainerNamespaceSerializer(serializers.ModelSerializer):
    my_permissions = MyPermissionsField(source='*', read_only=True)
    owners = serializers.SerializerMethodField()

    class Meta:
        model = models.ContainerNamespace
        fields = namespace_fields
        read_only_fields = ('name', 'my_permissions',)

    def get_owners(self, namespace):
        return get_users_with_perms(namespace, with_group_users=False).values_list(
            'username', flat=True)


class ContainerNamespaceDetailSerializer(ContainerNamespaceSerializer):
    groups = GroupPermissionField()

    class Meta:
        model = models.ContainerNamespace
        fields = namespace_fields + ('groups', )
        read_only_fields = ('name', 'my_permissions',)


class ContainerRepositorySerializer(serializers.ModelSerializer):
    pulp = serializers.SerializerMethodField()
    namespace = ContainerNamespaceSerializer()
    id = serializers.SerializerMethodField()
    created = serializers.SerializerMethodField()
    updated = serializers.SerializerMethodField()

    # This serializer is purposfully refraining from using pulp fields directly
    # in the top level response body. This is because future versions of hub will have to
    # support indexing other registries and the API responses for a container
    # repository should make sense for containers hosted by pulp and containers
    # hosted by other registries.
    class Meta:
        model = models.ContainerDistribution
        read_only_fields = (
            'id',
            'name',
            # this field will return null on instances where hub is indexing a
            # different repo
            'pulp',
            'namespace',
            'description',
            'created',
            'updated',
        )

        fields = read_only_fields

    def get_namespace(self, distro):
        return distro.namespace.name

    def get_id(self, distro):
        return distro.pulp_id

    def get_created(self, distro):
        return distro.repository.pulp_created

    def get_updated(self, distro):
        return distro.repository.pulp_last_updated

    def get_pulp(self, distro):
        repo = distro.repository

        return {
            'repository':
            {
                'pulp_id': repo.pulp_id,
                'pulp_type': repo.pulp_type,
                'version': repo.latest_version().number,
                'name': repo.name,
                'description': repo.description,
                'pulp_created': repo.pulp_created,
                'last_sync_task': _get_last_sync_task(repo),
                'pulp_labels': {
                    label.key: label.value for label in repo.pulp_labels.all()
                },
            },
            'distribution':
            {
                'pulp_id': distro.pulp_id,
                'name': distro.name,
                'pulp_created': distro.pulp_created,
                'base_path': distro.base_path,
                'pulp_labels': {
                    label.key: label.value for label in distro.pulp_labels.all()
                },
            }
        }


def _get_last_sync_task(repo):
    sync_task = models.container.ContainerSyncTask.objects.filter(
        repository=repo
    ).first()
    if not sync_task:
        # UI handles `null` as "no status"
        return

    return {
        "task_id": sync_task.id,
        "state": sync_task.task.state,
        "started_at": sync_task.task.started_at,
        "finished_at": sync_task.task.finished_at,
        "error": sync_task.task.error
    }


class ContainerManifestSerializer(serializers.ModelSerializer):
    config_blob = serializers.SerializerMethodField()
    tags = serializers.SerializerMethodField()
    layers = serializers.SerializerMethodField()

    class Meta:
        model = container_models.Manifest
        fields = (
            'pulp_id',
            'digest',
            'schema_version',
            'media_type',
            'config_blob',
            'tags',
            'pulp_created',
            'layers'
        )

    def get_layers(self, obj):
        layers = []
        # use the prefetched blob_list and artifact_list instead of obj.blobs and
        # blob._artifacts to cut down on queries made.
        for blob in obj.blob_list:
            layers.append({
                'digest': blob.digest,
                'size': blob.artifact_list[0].size
            })

        return layers

    def get_config_blob(self, obj):
        return {
            'digest': obj.config_blob.digest,
            'media_type': obj.config_blob.media_type
        }

    def get_tags(self, obj):
        tags = []
        # tagget_manifests returns all tags on the manifest, not just the ones
        # that are in the latest version of the repo.
        for tag in obj.tagged_manifests.all():
            tags.append(tag.name)

        return tags


class ContainerManifestDetailSerializer(ContainerManifestSerializer):
    def get_config_blob(self, obj):
        with obj.config_blob._artifacts.first().file.open() as f:
            config_json = json.load(f)

        return {
            'digest': obj.config_blob.digest,
            'media_type': obj.config_blob.media_type,
            'data': config_json
        }


class ContainerRepositoryHistorySerializer(serializers.ModelSerializer):
    added = serializers.SerializerMethodField()
    removed = serializers.SerializerMethodField()

    class Meta:
        model = core_models.RepositoryVersion
        fields = (
            'pulp_id',
            'added',
            'removed',
            'pulp_created',
            'number'
        )

    def get_added(self, obj):
        return [
            self._content_info(content.content) for content in obj.added_memberships.all()
        ]

    def get_removed(self, obj):
        return [
            self._content_info(content.content) for content in obj.removed_memberships.all()
        ]

    def _content_info(self, content):
        return_data = {
            "pulp_id": content.pulp_id,
            "pulp_type": content.pulp_type,
            "manifest_digest": None,
            "tag_name": None,
        }

        # TODO: Figure out if there is a way to prefetch Manifest and Tag objects
        if content.pulp_type == 'container.manifest':
            manifest = container_models.Manifest.objects.get(pk=content.pulp_id)
            return_data['manifest_digest'] = manifest.digest
        elif content.pulp_type == 'container.tag':
            tag = container_models.Tag.objects.select_related('tagged_manifest')\
                .get(pk=content.pulp_id)
            return_data['manifest_digest'] = tag.tagged_manifest.digest
            return_data['tag_name'] = tag.name

        return return_data


class ContainerReadmeSerializer(serializers.ModelSerializer):

    class Meta:
        model = models.ContainerDistroReadme
        fields = (
            'updated',
            'created',
            'text',
        )

        read_only_fields = (
            'updated',
            'created',
        )



# Generated by Django 2.2.15 on 2020-08-18 15:12

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('galaxy', '0004_update_synclist_policy_names'),
        ('galaxy', '0003_inbound_repo_per_namespace'),
    ]

    operations = [
    ]



from .collection import (
    CollectionArtifactDownloadView,
    CollectionImportViewSet,
    CollectionUploadViewSet,
    CollectionViewSet,
    CollectionVersionViewSet,
    CollectionVersionDocsViewSet,
    CollectionVersionMoveViewSet,
    UnpaginatedCollectionViewSet,
    UnpaginatedCollectionVersionViewSet,
    RepoMetadataViewSet,
)

from .namespace import (
    NamespaceViewSet,
)

from .task import (
    TaskViewSet,
)

from .sync import SyncConfigViewSet


__all__ = (
    'CollectionArtifactDownloadView',
    'CollectionImportViewSet',
    'CollectionUploadViewSet',
    'CollectionViewSet',
    'CollectionVersionViewSet',
    'CollectionVersionDocsViewSet',
    'CollectionVersionMoveViewSet',
    'NamespaceViewSet',
    'SyncConfigViewSet',
    'TaskViewSet',
    'UnpaginatedCollectionViewSet',
    'UnpaginatedCollectionVersionViewSet',
    'RepoMetadataViewSet',
)



import os

from django.core.management.base import BaseCommand, CommandError
from django.db import transaction
from django.utils.text import slugify
from pulp_ansible.app.models import (
    AnsibleDistribution,
    AnsibleRepository,
    CollectionRemote,
)
from pulp_ansible.app.tasks.utils import parse_collections_requirements_file
from rest_framework.serializers import ValidationError

from galaxy_ng.app.constants import COMMUNITY_DOMAINS


class Command(BaseCommand):
    """Django management command for creating new remotes.

    Example:

    $ create-remote rh-certified https://cloud.redhat.com/api/automation-hub/v3/collections

    $ create-remote community https://galaxy.ansible.com -r /tmp/reqs.yaml

    """

    status_messages = []

    def load_requirements_file(self, requirements_file):
        """Loads  and validates the requirements file

        If the requirements_file is an absolute path for a file it is
        opened and read.
        """
        if os.path.exists(requirements_file) and requirements_file.endswith((".yaml", ".yml")):
            with open(requirements_file) as file_obj:
                requirements_file = file_obj.read()

        try:
            parse_collections_requirements_file(requirements_file)
        except ValidationError as e:
            raise CommandError("Error parsing requirements_file {}".format(str(e)))
        else:
            self.status_messages.append("requirements_file loaded and parsed")

        return requirements_file

    def valid_url(self, url):
        if not url.endswith("/"):
            raise CommandError("url should end with '/'")
        return url

    def add_arguments(self, parser):
        # required positional arguments
        parser.add_argument('name', type=slugify, help="Remote Name")
        parser.add_argument(
            "url",
            type=self.valid_url,
            help="Remote Feed URL, Should end with '/'"
        )

        # optional named arguments
        parser.add_argument("--token", type=str, help="Remote Auth Token")
        parser.add_argument("--policy", type=str, help="Remote Download Policy")
        parser.add_argument("--auth_url", type=str, help="Remote Auth URL")
        parser.add_argument(
            "--repository",
            type=slugify,
            help="Name of a repository to create or associate, defaults to [name]"
        )
        parser.add_argument(
            "--distribution",
            type=slugify,
            help="Name of a distribution to create or associate, defaults to [name]"
        )
        parser.add_argument(
            "-r",
            "--requirements_file",
            type=self.load_requirements_file,
            help="Either an abs path ending in .yaml|.yml to be loaded or the literal YAML string."
        )

    def validate(self, data):
        if not data["requirements_file"] and any(
            [domain in data["url"] for domain in COMMUNITY_DOMAINS]
        ):
            raise CommandError(
                'Syncing content from community domains without specifying a '
                'requirements file is not allowed.'
            )
        return data

    def create_remote(self, data):
        remote, remote_created = CollectionRemote.objects.get_or_create(name=data["name"])

        for arg in ('url', 'auth_url', 'token', 'requirements_file'):
            if data[arg] is not None:
                setattr(remote, arg, data[arg])
        remote.save()
        self.status_messages.append(
            "{} CollectionRemote {}".format(
                "Created new" if remote_created else "Updated existing",
                remote.name
            )
        )

        return remote

    def create_repository(self, data, remote):
        repository, repo_created = AnsibleRepository.objects.get_or_create(
            name=data["repository"] or data["name"]
        )

        repository.remote = remote
        repository.save()
        self.status_messages.append(
            "{} Repository {}".format(
                "Created new" if repo_created else "Associated existing",
                repository.name,
            )
        )
        return repository

    def create_distribution(self, data, remote, repository):
        distro_name = data['distribution'] or data['name']
        distribution, distro_created = AnsibleDistribution.objects.get_or_create(name=distro_name)

        if not distribution.base_path:
            distribution.base_path = distro_name

        distribution.repository = repository
        distribution.remote = remote
        distribution.save()
        self.status_messages.append(
            "{} Distribution {}".format(
                "Created new" if distro_created else "Associated existing",
                distribution.name,
            )
        )
        return distribution

    def handle(self, *args, **data):
        data = self.validate(data)

        with transaction.atomic():
            remote = self.create_remote(data)
            repository = self.create_repository(data, remote)
            self.create_distribution(data, remote, repository)

        # only outputs success messages if transaction is succesful
        for message in self.status_messages:
            self.stdout.write(message)



import os

import boto3
from gunicorn import glogging
import logstash_formatter
import watchtower


AWS_ACCESS_KEY_ID = os.getenv('CW_AWS_ACCESS_KEY_ID')
AWS_SECRET_ACCESS_KEY = os.getenv('CW_AWS_SECRET_ACCESS_KEY')
AWS_REGION_NAME = os.getenv('CW_AWS_REGION_NAME')

LOGGING_GROUP = os.getenv('CW_LOGGING_GROUP')
LOGGING_STREAM_NAME = os.getenv('CW_LOGGING_STREAM_NAME')


class CloudWatchHandler(watchtower.CloudWatchLogHandler):
    """Pre-configured CloudWatch handler."""

    def __init__(self):
        boto3_session = boto3.Session(
            aws_access_key_id=AWS_ACCESS_KEY_ID,
            aws_secret_access_key=AWS_SECRET_ACCESS_KEY,
            region_name=AWS_REGION_NAME,
        )
        super().__init__(
            boto3_session=boto3_session,
            log_group=LOGGING_GROUP,
            stream_name=LOGGING_STREAM_NAME,
        )
        # TODO(cutwater): Request ID should not depend on Django framework
        # self.addFilter(request_id.logging.RequestIdFilter())
        self.setFormatter(logstash_formatter.LogstashFormatterV1())


class GunicornLogger(glogging.Logger):

    def setup(self, cfg):
        super().setup(cfg)

        self.error_log.addHandler(CloudWatchHandler())
        self.access_log.addHandler(CloudWatchHandler())



import logging

from pulp_ansible.app.models import (
    AnsibleDistribution,
    CollectionVersion,
)
from rest_framework import serializers
import semantic_version

from .base import Serializer
from galaxy_ng.app.api.v3.serializers.namespace import NamespaceSummarySerializer
from galaxy_ng.app.models import Namespace

log = logging.getLogger(__name__)


class ContentSummarySerializer(Serializer):

    def to_representation(self, contents):
        summary = {"role": [], "module": [], "playbook": [], "plugin": []}
        for item in contents:
            key = self._get_content_type_key(item["content_type"])
            summary[key].append(item['name'])
        return {"total_count": sum(map(len, summary.items())), "contents": summary}

    @staticmethod
    def _get_content_type_key(content_type: str) -> str:
        # FIXME(cutwater): Replace with galaxy-importer constants usage
        if content_type == "role":  # ContentType.ROLE (from galaxy-importer)
            return "role"
        elif content_type == "module":  # ContentType.MODULE (from galaxy-importer)
            return "module"
        elif content_type == "playbook":  # ContentType.PLAYBOOK (from galaxy-importer)
            return "playbook"
        else:
            return "plugin"


class ContentSerializer(Serializer):
    name = serializers.CharField()
    content_type = serializers.CharField()
    description = serializers.CharField()


class CollectionMetadataSerializer(Serializer):
    dependencies = serializers.JSONField()
    contents = serializers.JSONField()

    # URLs
    documentation = serializers.CharField()
    homepage = serializers.CharField()
    issues = serializers.CharField()
    repository = serializers.CharField()

    description = serializers.CharField()
    authors = serializers.ListField(serializers.CharField())
    license = serializers.ListField(serializers.CharField())
    tags = serializers.SerializerMethodField()

    def get_tags(self, collection_version):
        # TODO(awcrosby): remove when galaxy_pulp no longer used in _ui
        if isinstance(collection_version, dict):
            return [tag['name'] for tag in collection_version['tags']]

        return [tag.name for tag in collection_version.tags.all()]


class CollectionVersionBaseSerializer(Serializer):
    namespace = serializers.CharField()
    name = serializers.CharField()
    version = serializers.CharField()
    requires_ansible = serializers.CharField()
    created_at = serializers.DateTimeField(source='pulp_created')
    metadata = CollectionMetadataSerializer(source='*')
    contents = serializers.ListField(ContentSerializer())


class CollectionVersionSerializer(CollectionVersionBaseSerializer):
    repository_list = serializers.SerializerMethodField()

    def get_repository_list(self, collection_version):
        """Repository list where content is in the latest RepositoryVersion."""

        # get all repos where content exists in a RepositoryVersion
        content = collection_version.content_ptr
        all_repos = content.repositories.all().distinct().exclude(
            name__startswith='inbound-').exclude(name__endswith='-synclist')

        qs = CollectionVersion.objects.filter(pk=collection_version.pk)
        cv_in_repo_latest_version = []
        for repo in all_repos:
            if qs.filter(pk__in=repo.latest_version().content):
                cv_in_repo_latest_version.append(repo.name)
        return cv_in_repo_latest_version


class CollectionVersionDetailSerializer(CollectionVersionBaseSerializer):
    docs_blob = serializers.JSONField()


class CollectionVersionSummarySerializer(Serializer):
    version = serializers.CharField()
    created = serializers.CharField(source='pulp_created')


class _CollectionSerializer(Serializer):
    """ Serializer for pulp_ansible CollectionViewSet.
    Uses CollectionVersion object to serialize associated Collection data.
    """

    id = serializers.UUIDField(source='pk')
    namespace = serializers.SerializerMethodField()
    name = serializers.CharField()
    download_count = serializers.IntegerField(default=0)
    latest_version = serializers.SerializerMethodField()

    def get_namespace(self, obj):
        namespace = Namespace.objects.get(name=obj.namespace)
        return NamespaceSummarySerializer(namespace).data


class CollectionListSerializer(_CollectionSerializer):
    deprecated = serializers.BooleanField()

    def get_latest_version(self, obj):
        return CollectionVersionBaseSerializer(obj).data


class CollectionDetailSerializer(_CollectionSerializer):
    all_versions = serializers.SerializerMethodField()

    # TODO: rename field to "version_details" since with
    # "version" query param this won't always be the latest version
    def get_latest_version(self, obj):
        return CollectionVersionDetailSerializer(obj).data

    def get_all_versions(self, obj):
        path = self.context['request'].parser_context['kwargs']['path']
        distro = AnsibleDistribution.objects.get(base_path=path)
        repository_version = distro.repository.latest_version()
        versions_in_repo = CollectionVersion.objects.filter(
            pk__in=repository_version.content,
            collection=obj.collection,
        )
        versions_in_repo = sorted(
            versions_in_repo, key=lambda obj: semantic_version.Version(obj.version), reverse=True
        )
        return CollectionVersionSummarySerializer(versions_in_repo, many=True).data



import logging
from rest_framework import status
from django.urls import reverse
from django.test import override_settings
from pulp_ansible.app.models import CollectionRemote
from galaxy_ng.app.constants import DeploymentMode
from .base import BaseTestCase

log = logging.getLogger(__name__)


@override_settings(GALAXY_DEPLOYMENT_MODE=DeploymentMode.STANDALONE.value)
class TestUiTaskListViewSet(BaseTestCase):
    def setUp(self):
        super().setUp()

        self.admin_user = self._create_user("admin")
        self.pe_group = self._create_partner_engineer_group()
        self.admin_user.groups.add(self.pe_group)
        self.admin_user.save()

        self.certified_remote = CollectionRemote.objects.get(name='rh-certified')

    def build_sync_url(self, path):
        return reverse('galaxy:api:content:v3:sync', kwargs={'path': path})

    def build_task_url(self):
        return reverse('galaxy:api:v3:default-content:tasks-list')

    def build_task_detail_url(self, pk):
        return reverse('galaxy:api:v3:default-content:tasks-detail', kwargs={"pk": pk})

    def test_tasks_required_fields(self):
        # Spawn a task
        self.client.force_authenticate(user=self.admin_user)
        response = self.client.post(self.build_sync_url(self.certified_remote.name))
        log.debug('Spawn a task for testing tasks/ endpoint')
        log.debug('response: %s', response)
        log.debug('response.data: %s', response.data)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertIn('task', response.data)

        # Ensure all required fields are present in list URL
        response = self.client.get(self.build_task_url())
        log.debug('response: %s', response)
        log.debug('response.data: %s', response.data)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertGreater(response.data['meta']['count'], 0)
        required_fields = (
            'pulp_id',
            'name',
            'finished_at',
            'started_at',
            'state',
            'repository',
            'href',
        )
        for field in required_fields:
            self.assertIn(field, response.data['data'][0])

        # Test Detail URL
        task_pk = response.data['data'][0]['pulp_id']
        response = self.client.get(self.build_task_detail_url(task_pk))
        log.debug('response: %s', response)
        log.debug('response.data: %s', response.data)
        required_fields = (
            'pulp_id',
            'name',
            'created_at',
            'updated_at',
            'finished_at',
            'started_at',
            'state',
            'error',
            'worker',
            'parent_task',
            'child_tasks',
            'repository',
            'progress_reports',
        )
        for field in required_fields:
            self.assertIn(field, response.data)



from .publishing import import_and_move_to_staging, import_and_auto_approve  # noqa: F401
from .promotion import call_copy_task, call_remove_task  # noqa: F401
from .synclist import curate_synclist_repository, curate_all_synclist_repository  # noqa: F401
# from .synchronizing import synchronize  # noqa



import logging

from django.db.models import Prefetch, Count, Q
from django.core import exceptions

from pulpcore.plugin import models as core_models
from pulp_container.app import models as container_models

from django_filters import filters
from django_filters.rest_framework import filterset, DjangoFilterBackend

from guardian.shortcuts import get_objects_for_user

from galaxy_ng.app.api import base as api_base
from galaxy_ng.app.api.ui import serializers
from django.shortcuts import get_object_or_404
from galaxy_ng.app.access_control import access_policy

from galaxy_ng.app import models

log = logging.getLogger(__name__)


class RepositoryFilter(filterset.FilterSet):
    my_permissions = filters.CharFilter(method='has_permissions')

    sort = filters.OrderingFilter(
        fields=(
            ('repository__pulp_created', 'created'),
            ('name', 'name'),
            ('description', 'description'),
            ('repository__pulp_last_updated', 'updated'),
        ),
    )

    class Meta:
        model = models.ContainerDistribution
        fields = {
            'name': ['exact', 'icontains', 'contains', 'startswith'],
            'description': ['exact', 'icontains', 'contains', 'startswith'],
        }

    def has_permissions(self, queryset, name, value):
        perms = self.request.query_params.getlist(name)
        namespaces = get_objects_for_user(
            self.request.user, perms, klass=container_models.ContainerNamespace)
        return self.queryset.filter(namespace__in=namespaces)


class ManifestFilter(filterset.FilterSet):
    sort = filters.OrderingFilter(
        fields=(
            ('pulp_created', 'created'),
        ),
    )

    class Meta:
        model = container_models.Manifest
        # Tag filters are supported, but are done in get_queryset. See the comment
        # there
        fields = {
            'digest': ['exact', 'icontains', 'contains', 'startswith'],
        }


class HistoryFilter(filterset.FilterSet):
    sort = filters.OrderingFilter(
        fields=(
            ('pulp_created', 'created'),
            ('number', 'number'),
        ),
    )


class ContainerNamespaceViewSet(api_base.ModelViewSet):
    queryset = models.ContainerNamespace.objects.all()
    serializer_class = serializers.ContainerNamespaceDetailSerializer
    permission_classes = [access_policy.ContainerNamespaceAccessPolicy]
    lookup_field = "name"


class ContainerRepositoryViewSet(api_base.ModelViewSet):
    queryset = models.ContainerDistribution.objects.all().select_related('namespace')
    serializer_class = serializers.ContainerRepositorySerializer
    filter_backends = (DjangoFilterBackend,)
    filterset_class = RepositoryFilter
    permission_classes = [access_policy.ContainerRepositoryAccessPolicy]
    lookup_field = "base_path"


# provide some common methods across all <distro>/_content/ endpoints
class ContainerContentBaseViewset(api_base.ModelViewSet):
    permission_classes = [access_policy.ContainerRepositoryAccessPolicy]

    def get_distro(self):
        return get_object_or_404(
            models.ContainerDistribution, base_path=self.kwargs["base_path"])


class ContainerRepositoryManifestViewSet(ContainerContentBaseViewset):
    permission_classes = [access_policy.ContainerRepositoryAccessPolicy]
    filter_backends = (DjangoFilterBackend,)
    filterset_class = ManifestFilter

    def get_serializer_class(self):
        if self.action == 'retrieve':
            return serializers.ContainerManifestDetailSerializer
        return serializers.ContainerManifestSerializer

    def get_queryset(self):
        repo = self.get_distro().repository
        repo_version = repo.latest_version()
        repo_content = repo_version.content.all()

        manifests = (
            repo_version.get_content(container_models.Manifest.objects)
            .prefetch_related(
                # Prefetch limits the tag list to tags in the current repo version.
                # Without it, tags from all repo versions are allowed which leads
                # to duplicates.
                Prefetch(
                    'tagged_manifests',
                    container_models.Tag.objects.filter(pk__in=repo_content)),
                Prefetch(
                    'blobs',
                    container_models.Blob.objects.prefetch_related(
                        Prefetch('_artifacts', to_attr='artifact_list')),
                    to_attr='blob_list'),
                'config_blob'
            )
        )

        # I know that this should go in the FilterSet, but I cannot for the life
        # of me figure out how to access base_path in the filterset. Without
        # base_path, it's impossible to filter the tags down to a specific repo version,
        # which means this query would end up pulling all of the tags in all repos
        # that match the tag param, which could potentially be thousands of objects.
        tag_filter = self.request.GET.get('tag', None)
        if tag_filter:
            manifests = manifests.filter(
                # tagged_manifests doesn't respect the Prefetch filtering, so
                # the repo version has to be filtered again here
                tagged_manifests__pk__in=repo_version.get_content(
                    container_models.Tag.objects).filter(name__icontains=tag_filter))

        return manifests

    def get_object(self):
        qs = self.get_queryset()
        # manifest_ref can be a tag name or a manifest digest
        manifest_ref = self.kwargs['manifest_ref']
        try:
            repo_version = self.get_distro().repository.latest_version()

            # we could just return tag.tagged_manifest, but using the tag in the
            # queryset allows this to take advantage of all the prefetching that
            # happens in get_queryset
            tag = repo_version.get_content(container_models.Tag.objects).get(name=manifest_ref)
            manifest = qs.get(tagged_manifests__pk=tag)
        except exceptions.ObjectDoesNotExist:
            manifest = get_object_or_404(qs, digest=manifest_ref)

        return manifest


class ContainerRepositoryHistoryViewSet(ContainerContentBaseViewset):
    serializer_class = serializers.ContainerRepositoryHistorySerializer
    filter_backends = (DjangoFilterBackend,)
    filterset_class = HistoryFilter

    def get_queryset(self):
        repo = self.get_distro().repository

        allowed_content_types = ['container.manifest', 'container.tag']

        # The ui only cares about repo versions where tags and manifests are added.
        # Pulp container revs the repo version each time any blobs are added, so
        # this filters out any repo versions where tags and manifests are unchanged.
        return (
            repo.versions.annotate(
                # Count the number of added/removed manifests and tags and use
                # the result to filter out any versions where tags and manifests
                # are unchanged.
                added_count=Count('added_memberships', filter=Q(
                    added_memberships__content__pulp_type__in=allowed_content_types)),
                removed_count=Count('removed_memberships', filter=Q(
                    removed_memberships__content__pulp_type__in=allowed_content_types))
            )
            .filter(Q(added_count__gt=0) | Q(removed_count__gt=0))
            .prefetch_related(
                Prefetch(
                    'added_memberships',
                    queryset=core_models.RepositoryContent.objects.filter(
                        content__pulp_type__in=allowed_content_types).select_related('content'),
                ),
                Prefetch(
                    'removed_memberships',
                    queryset=core_models.RepositoryContent.objects.filter(
                        content__pulp_type__in=allowed_content_types).select_related('content'),
                )
            ).order_by('-pulp_created')
        )


class ContainerReadmeViewSet(ContainerContentBaseViewset):
    queryset = models.ContainerDistroReadme.objects
    serializer_class = serializers.ContainerReadmeSerializer
    permission_classes = [access_policy.ContainerReadmeAccessPolicy]

    def get_object(self):
        distro = self.get_distro()
        return self.queryset.get_or_create(container=distro)[0]



from rest_framework import mixins
from pulp_ansible.app import models as pulp_models
from guardian.shortcuts import get_objects_for_user

from galaxy_ng.app.access_control import access_policy
from galaxy_ng.app.api.ui import serializers, versioning
from galaxy_ng.app.api import base as api_base
from galaxy_ng.app import models


class DistributionViewSet(
    mixins.ListModelMixin,
    mixins.RetrieveModelMixin,
    api_base.GenericViewSet,
):
    serializer_class = serializers.DistributionSerializer
    model = pulp_models.AnsibleDistribution
    queryset = pulp_models.AnsibleDistribution.objects.exclude(
        name__startswith='inbound-').exclude(
            name__endswith='-synclist').order_by('name')
    permission_classes = [access_policy.DistributionAccessPolicy]
    versioning_class = versioning.UIVersioning


class MyDistributionViewSet(DistributionViewSet):
    permission_classes = [access_policy.MyDistributionAccessPolicy]

    def get_queryset(self):
        synclists = get_objects_for_user(
            self.request.user,
            'galaxy.change_synclist',
            any_perm=True,
            accept_global_perms=False,
            klass=models.SyncList
        )

        # TODO: find a better way query this data
        return pulp_models.AnsibleDistribution.objects.filter(
            name__in=synclists.values_list('name', flat=True)).order_by('name')



from .namespace import (
    NamespaceViewSet,
)

from .collection import (
    CollectionViewSet,
    CollectionVersionViewSet,
    CollectionImportViewSet,
    CollectionRemoteViewSet
)
from .my_namespace import MyNamespaceViewSet
from .my_synclist import MySyncListViewSet
from .tags import TagsViewSet
from .user import UserViewSet, CurrentUserViewSet
from .synclist import SyncListViewSet
from .root import APIRootView
from .group import GroupViewSet, GroupModelPermissionViewSet, GroupUserViewSet
from .distribution import DistributionViewSet, MyDistributionViewSet
from .execution_environment import (
    ContainerRepositoryViewSet,
    ContainerRepositoryManifestViewSet,
    ContainerRepositoryHistoryViewSet,
    ContainerReadmeViewSet,
    ContainerNamespaceViewSet
)

__all__ = (
    'NamespaceViewSet',
    'MyNamespaceViewSet',
    'MySyncListViewSet',
    'CollectionViewSet',
    'CollectionVersionViewSet',
    'CollectionImportViewSet',
    'CollectionRemoteViewSet',
    'TagsViewSet',
    'CurrentUserViewSet',
    'UserViewSet',
    'SyncListViewSet',
    'APIRootView',
    'GroupViewSet',
    'GroupModelPermissionViewSet',
    'GroupUserViewSet',
    'DistributionViewSet',
    'MyDistributionViewSet',
    'ContainerRepositoryViewSet',
    'ContainerRepositoryManifestViewSet',
    'ContainerRepositoryHistoryViewSet',
    'ContainerReadmeViewSet',
    'ContainerNamespaceViewSet'
)



from pulpcore.plugin.tasking import enqueue_with_reservation
from pulp_ansible.app.models import AnsibleRepository, CollectionVersion
from pulp_ansible.app.tasks.copy import copy_content


def call_copy_task(collection_version, source_repo, dest_repo):
    """Calls pulp_ansible task to copy content from source to destination repo."""
    locks = [source_repo, dest_repo]
    config = [{
        'source_repo_version': source_repo.latest_version().pk,
        'dest_repo': dest_repo.pk,
        'content': [collection_version.pk],
    }]
    return enqueue_with_reservation(
        copy_content,
        locks,
        args=[config],
        kwargs={},
    )


def call_remove_task(collection_version, repository):
    """Calls task to remove content from repo."""
    remove_task_args = (collection_version.pk, repository.pk)
    return enqueue_with_reservation(
        _remove_content_from_repository,
        [repository],
        args=remove_task_args,
        kwargs={},
    )


def _remove_content_from_repository(collection_version_pk, repository_pk):
    """
    Remove a CollectionVersion from a repository.
    Args:
        collection_version_pk: The pk of the CollectionVersion to remove from repository.
        repository_pk: The pk of the AnsibleRepository to remove the CollectionVersion from.
    """
    repository = AnsibleRepository.objects.get(pk=repository_pk)
    qs = CollectionVersion.objects.filter(pk=collection_version_pk)
    with repository.new_version() as new_version:
        new_version.remove_content(qs)



from django.core.exceptions import PermissionDenied
from django.http import Http404

from rest_framework import exceptions
from rest_framework.response import Response
from rest_framework.settings import api_settings


def _get_errors(detail, *, status, title, source=None):
    if isinstance(detail, list):
        for item in detail:
            yield from _get_errors(item, status=status, title=title, source=source)
    elif isinstance(detail, dict):
        for key, value in detail.items():
            yield from _get_errors(value, status=status, title=title, source=key)
    else:
        error = {
            'status': str(status),
            'code': detail.code,
            'title': title,
        }

        if title != detail:
            error['detail'] = str(detail)
        if source and source != api_settings.NON_FIELD_ERRORS_KEY:
            error['source'] = {'parameter': source}

        yield error


def _handle_drf_api_exception(exc):
    headers = {}
    if getattr(exc, 'auth_header', None):
        headers['WWW-Authenticate'] = exc.auth_header
    if getattr(exc, 'wait', None):
        headers['Retry-After'] = '%d' % exc.wait

    title = exc.__class__.default_detail
    errors = _get_errors(exc.detail, status=exc.status_code, title=title)
    data = {'errors': list(errors)}
    return Response(data, status=exc.status_code, headers=headers)


def exception_handler(exc, context):
    """Custom exception handler."""

    if isinstance(exc, Http404):
        exc = exceptions.NotFound()
    elif isinstance(exc, PermissionDenied):
        exc = exceptions.PermissionDenied()

    if isinstance(exc, exceptions.APIException):
        return _handle_drf_api_exception(exc)

    return None



from rest_framework import serializers

from pulp_ansible.app import viewsets as pulp_viewsets
from pulp_ansible.app.models import (
    AnsibleDistribution,
    AnsibleRepository,
    CollectionRemote,
)

from galaxy_ng.app.constants import COMMUNITY_DOMAINS
from galaxy_ng.app.models.collectionsync import CollectionSyncTask


class AnsibleDistributionSerializer(serializers.ModelSerializer):
    created_at = serializers.DateTimeField(source='pulp_created')
    updated_at = serializers.DateTimeField(source='pulp_last_updated')

    class Meta:
        model = AnsibleDistribution
        fields = (
            'name',
            'base_path',
            'content_guard',
            'created_at',
            'updated_at',
        )


class LastSyncTaskMixin:

    def get_last_sync_task_queryset(self, obj):
        raise NotImplementedError("subclass must implement get_last_sync_task_queryset")

    def get_last_sync_task(self, obj):
        sync_task = self.get_last_sync_task_queryset(obj)
        if not sync_task:
            # UI handles `null` as "no status"
            return

        return {
            "task_id": sync_task.id,
            "state": sync_task.task.state,
            "started_at": sync_task.task.started_at,
            "finished_at": sync_task.task.finished_at,
            "error": sync_task.task.error
        }


class AnsibleRepositorySerializer(LastSyncTaskMixin, serializers.ModelSerializer):
    distributions = serializers.SerializerMethodField()
    last_sync_task = serializers.SerializerMethodField()
    created_at = serializers.DateTimeField(source='pulp_created')
    updated_at = serializers.DateTimeField(source='pulp_last_updated')

    class Meta:
        model = AnsibleRepository
        fields = (
            'name',
            'description',
            'next_version',
            'distributions',
            'created_at',
            'updated_at',
            'last_sync_task',
        )

    def get_distributions(self, obj):
        return [
            AnsibleDistributionSerializer(distro).data
            for distro in obj.ansible_ansibledistribution.all()
        ]

    def get_last_sync_task_queryset(self, obj):
        return CollectionSyncTask.objects.filter(repository=obj).first()


class CollectionRemoteSerializer(LastSyncTaskMixin, pulp_viewsets.CollectionRemoteSerializer):
    last_sync_task = serializers.SerializerMethodField()
    write_only_fields = serializers.SerializerMethodField()
    created_at = serializers.DateTimeField(source='pulp_created', required=False)
    updated_at = serializers.DateTimeField(source='pulp_last_updated', required=False)
    proxy_password = serializers.CharField(
        help_text="Password for proxy authentication.",
        allow_null=True,
        required=False,
        style={'input_type': 'password'},
        write_only=True
    )
    proxy_username = serializers.CharField(
        help_text="User for proxy authentication.",
        allow_null=True,
        required=False,
        write_only=False,  # overwriting this as pulpcore defaults to True
    )
    token = serializers.CharField(
        allow_null=True,
        required=False,
        max_length=2000,
        write_only=True,
        style={'input_type': 'password'}
    )
    password = serializers.CharField(
        help_text="Remote password.",
        allow_null=True,
        required=False,
        style={'input_type': 'password'},
        write_only=True
    )
    username = serializers.CharField(
        help_text="Remote user.",
        allow_null=True,
        required=False,
        write_only=False,  # overwriting this as pulpcore defaults to True
    )
    name = serializers.CharField(read_only=True)
    repositories = serializers.SerializerMethodField()

    class Meta:
        model = CollectionRemote
        fields = (
            'pk',
            'name',
            'url',
            'auth_url',
            'token',
            'policy',
            'requirements_file',
            'created_at',
            'updated_at',
            'username',
            'password',
            'tls_validation',
            'client_key',
            'client_cert',
            'ca_cert',
            'last_sync_task',
            'repositories',
            'pulp_href',
            'download_concurrency',
            'proxy_url',
            'proxy_username',
            'proxy_password',
            'write_only_fields',
            'rate_limit'
        )
        extra_kwargs = {
            'name': {'read_only': True},
            'pulp_href': {'read_only': True},
            'client_key': {'write_only': True},
        }

    def validate(self, data):
        if not data.get('requirements_file') and any(
            [domain in data['url'] for domain in COMMUNITY_DOMAINS]
        ):
            raise serializers.ValidationError(
                detail={
                    'requirements_file':
                        'Syncing content from community domains without specifying a '
                        'requirements file is not allowed.'
                }
            )
        return super().validate(data)

    def get_write_only_fields(self, obj):
        return get_write_only_fields(self, obj)

    def get_repositories(self, obj):
        return [
            AnsibleRepositorySerializer(repo).data
            for repo in obj.repository_set.all()
        ]

    def get_last_sync_task_queryset(self, obj):
        """Gets last_sync_task from Pulp using remote->repository relation"""

        return CollectionSyncTask.objects.filter(
            repository=obj.repository_set.order_by('-pulp_last_updated').first()
        ).first()


def get_write_only_fields(serializer, obj, extra_data=None):
    """
    Returns a list of write only fields and whether or not their values are set
    so that clients can tell if they are overwriting an existing value.
    serializer: Serializer instance
    obj: model object being serialized
    extra_data: extra fields that might not be on obj. This is used when a write
        only field is not one of the fields in the underlying data model.
    """
    fields = []
    extra_data = extra_data or {}

    # returns false if field is "" or None
    def _is_set(field_name):
        if (field_name in extra_data):
            return bool(extra_data[field_name])
        else:
            return bool(getattr(obj, field_name))

    # There are two ways to set write_only. This checks both.

    # check for values that are set to write_only in Meta.extra_kwargs
    for field_name in serializer.Meta.extra_kwargs:
        if serializer.Meta.extra_kwargs[field_name].get('write_only', False):
            fields.append({"name": field_name, "is_set": _is_set(field_name)})

    # check for values that are set to write_only in fields
    serializer_fields = serializer.get_fields()
    for field_name in serializer_fields:
        if (serializer_fields[field_name].write_only):
            fields.append({"name": field_name, "is_set": _is_set(field_name)})

    return fields



from django.contrib import auth as django_auth
from django.utils.decorators import method_decorator
from django.views.decorators.csrf import ensure_csrf_cookie
from rest_framework.authentication import SessionAuthentication

from rest_framework.response import Response
from rest_framework import status as http_code

from galaxy_ng.app.api import base as api_base
from galaxy_ng.app.access_control import access_policy


from galaxy_ng.app.api.ui.serializers import LoginSerializer


__all__ = (
    "LoginView",
    "LogoutView",
)


class _CSRFSessionAuthentication(SessionAuthentication):
    """CSRF-enforcing version of a SessionAuthentication class."""

    def authenticate(self, request):
        user = getattr(request._request, "user", None)
        self.enforce_csrf(request)
        return user, None


class LoginView(api_base.GenericAPIView):
    serializer_class = LoginSerializer
    authentication_classes = (_CSRFSessionAuthentication,)
    permission_classes = [access_policy.LoginAccessPolicy]

    @method_decorator(ensure_csrf_cookie)
    def get(self, request, *args, **kwargs):
        return Response(status=http_code.HTTP_204_NO_CONTENT)

    def post(self, request, *args, **kwargs):
        serializer = self.get_serializer(data=self.request.data)
        serializer.is_valid(raise_exception=True)

        username = serializer.validated_data['username']
        password = serializer.validated_data['password']

        user = django_auth.authenticate(request, username=username, password=password)
        if user is None:
            return Response(status=http_code.HTTP_403_FORBIDDEN)

        django_auth.login(request, user)
        return Response(status=http_code.HTTP_204_NO_CONTENT)


class LogoutView(api_base.APIView):
    permission_classes = [access_policy.LogoutAccessPolicy]

    def post(self, request, *args, **kwargs):
        django_auth.logout(request)
        return Response(status=http_code.HTTP_204_NO_CONTENT)



"""Constants for Pulp Galaxy plugin tests."""
from urllib.parse import urljoin

from pulp_smash.constants import PULP_FIXTURES_BASE_URL
from pulp_smash.pulp3.constants import (
    BASE_DISTRIBUTION_PATH,
    BASE_PUBLICATION_PATH,
    BASE_REMOTE_PATH,
    BASE_REPO_PATH,
    BASE_CONTENT_PATH,
)

# FIXME: list any download policies supported by your plugin type here.
# If your plugin supports all download policies, you can import this
# from pulp_smash.pulp3.constants instead.
# DOWNLOAD_POLICIES = ["immediate", "streamed", "on_demand"]
DOWNLOAD_POLICIES = ["immediate"]

# FIXME: replace 'unit' with your own content type names, and duplicate as necessary for each type
GALAXY_CONTENT_NAME = "galaxy.unit"

# FIXME: replace 'unit' with your own content type names, and duplicate as necessary for each type
GALAXY_CONTENT_PATH = urljoin(BASE_CONTENT_PATH, "galaxy/units/")

GALAXY_REMOTE_PATH = urljoin(BASE_REMOTE_PATH, "galaxy/galaxy/")

GALAXY_REPO_PATH = urljoin(BASE_REPO_PATH, "galaxy/galaxy/")

GALAXY_PUBLICATION_PATH = urljoin(BASE_PUBLICATION_PATH, "galaxy/galaxy/")

GALAXY_DISTRIBUTION_PATH = urljoin(BASE_DISTRIBUTION_PATH, "galaxy/galaxy/")

# FIXME: replace this with your own fixture repository URL and metadata
GALAXY_FIXTURE_URL = urljoin(PULP_FIXTURES_BASE_URL, "galaxy/")
"""The URL to a galaxy repository."""

# FIXME: replace this with the actual number of content units in your test fixture
GALAXY_FIXTURE_COUNT = 3
"""The number of content units available at :data:`GALAXY_FIXTURE_URL`."""

GALAXY_FIXTURE_SUMMARY = {GALAXY_CONTENT_NAME: GALAXY_FIXTURE_COUNT}
"""The desired content summary after syncing :data:`GALAXY_FIXTURE_URL`."""

# FIXME: replace this with the location of one specific content unit of your choosing
GALAXY_URL = urljoin(GALAXY_FIXTURE_URL, "")
"""The URL to an galaxy file at :data:`GALAXY_FIXTURE_URL`."""

# FIXME: replace this with your own fixture repository URL and metadata
GALAXY_INVALID_FIXTURE_URL = urljoin(PULP_FIXTURES_BASE_URL, "galaxy-invalid/")
"""The URL to an invalid galaxy repository."""

# FIXME: replace this with your own fixture repository URL and metadata
GALAXY_LARGE_FIXTURE_URL = urljoin(PULP_FIXTURES_BASE_URL, "galaxy_large/")
"""The URL to a galaxy repository containing a large number of content units."""

# FIXME: replace this with the actual number of content units in your test fixture
GALAXY_LARGE_FIXTURE_COUNT = 25
"""The number of content units available at :data:`GALAXY_LARGE_FIXTURE_URL`."""



from .auth import (
    Group,
    User,
)
from .collectionimport import (
    CollectionImport,
)
from .namespace import (
    Namespace,
    NamespaceLink,
)

from .synclist import (
    SyncList,
)

from .collectionsync import (
    CollectionSyncTask
)

from .container import (
    ContainerDistribution,
    ContainerDistroReadme,
    ContainerNamespace
)

__all__ = (
    'Group',
    'User',
    'CollectionImport',
    'Namespace',
    'NamespaceLink',
    'SyncList',
    'CollectionSyncTask',
    'ContainerDistribution',
    'ContainerDistroReadme',
    'ContainerNamespace'
)



from django.db import migrations

STAGING = 'staging'
REJECTED = 'rejected'

def add_repos_staging_rejected(apps, schema_editor):
    AnsibleRepository = apps.get_model('ansible', 'AnsibleRepository')
    AnsibleDistribution = apps.get_model('ansible', 'AnsibleDistribution')
    RepositoryVersion = apps.get_model('core', 'RepositoryVersion')
    db_alias = schema_editor.connection.alias

    def create_repo_and_distro(name):
        repo = AnsibleRepository.objects.using(db_alias).create(
            name=name,
            pulp_type='ansible.ansible',
            next_version=1,
        )
        RepositoryVersion.objects.using(db_alias).create(
            repository=repo,
            number=0,
            complete=True,
        )
        AnsibleDistribution.objects.using(db_alias).create(
            name=name,
            base_path=name,
            repository=repo,
            pulp_type='ansible.ansible',
        )

    if not AnsibleRepository.objects.using(db_alias).filter(name=STAGING):
        create_repo_and_distro(STAGING)

    if not AnsibleRepository.objects.using(db_alias).filter(name=REJECTED):
        create_repo_and_distro(REJECTED)


class Migration(migrations.Migration):

    dependencies = [
        ('galaxy', '0009_add_repoversion_to_inbound_repos'),
    ]

    operations = [
        migrations.RunPython(add_repos_staging_rejected),
    ]



# Generated by Django 2.2.19 on 2021-03-24 22:07

from django.db import migrations

REPOSITORIES = [
    {
        "name": "published",
        "description": "Certified content repository",
        "pulp_type": "ansible.ansible",
        "next_version": 1,
    },
    {
        "name": "community",
        "description": "Community content repository",
        "pulp_type": "ansible.ansible",
        "next_version": 1,
        "remote": {
            "name": "community",
            "url": "https://galaxy.ansible.com/api/",
            "requirements_file": None,
            "token": None,
            "auth_url": None,
            "pulp_type": "ansible.ansible",
            "rate_limit": 8,
        }
    },
    {
        "name": "rh-certified",
        "description": "Red Hat certified content repository",
        "pulp_type": "ansible.ansible",
        "next_version": 1,
        "remote": {
            "name": "rh-certified",
            "url": "https://cloud.redhat.com/api/automation-hub/",
            "requirements_file": None,
            "token": None,
            "auth_url": (
                "https://sso.redhat.com/auth/realms/"
                "redhat-external/protocol/openid-connect/token"
            ),
            "pulp_type": "ansible.ansible",
            "rate_limit": 8,
        }
    }
]
# NOTE:`staging` & `rejected` are created by 0010_add_staging_rejected_repos.py


def populate_initial_repos(apps, schema_editor):
    """Populates initial data for Automation Hub.

    - Staging and rejected repos are covered by migration 0010
    - Remotes are created only if doesn't exist and is specified in REMOTES dict
      matching the same name as repository.
    - Repositories are created using REPOSITORIES data and only if doesn't exist.
    - Distributions are created only if not found with same name as repository.
    - Reverse us NOOP so nothing is done if transaction fails to avoid corrupting
      existing data.

    """

    db_alias = schema_editor.connection.alias

    AnsibleRepository = apps.get_model('ansible', 'AnsibleRepository')
    AnsibleRepository.objects = AnsibleRepository.objects.using(db_alias)

    AnsibleDistribution = apps.get_model('ansible', 'AnsibleDistribution')
    AnsibleDistribution.objects = AnsibleDistribution.objects.using(db_alias)

    CollectionRemote = apps.get_model('ansible', 'CollectionRemote')
    CollectionRemote.objects = CollectionRemote.objects.using(db_alias)

    RepositoryVersion = apps.get_model('core', 'RepositoryVersion')
    RepositoryVersion.objects = RepositoryVersion.objects.using(db_alias)

    for repo_data in REPOSITORIES:

        remote = repo_data.pop("remote", None)
        if remote is not None:
            remote, _ = CollectionRemote.objects.get_or_create(
                name=remote["name"], defaults=remote
            )
        repo_data["remote"] = remote

        repository, _ = AnsibleRepository.objects.get_or_create(
            name=repo_data["name"],
            defaults=repo_data
        )

        if not RepositoryVersion.objects.filter(repository__name=repository.name):
            RepositoryVersion.objects.create(
                repository=repository,
                number=0,
                complete=True
            )

        AnsibleDistribution.objects.get_or_create(
            base_path=repository.name,
            defaults={
                "name": repository.name,
                "base_path": repository.name,
                "remote": remote,
                "repository": repository,
                "pulp_type": "ansible.ansible"
            }
        )


class Migration(migrations.Migration):

    dependencies = [
        ('galaxy', '0016_containerregistryremote_containerregistryrepos_containersynctask'),
    ]

    operations = [
        migrations.RunPython(
            code=populate_initial_repos,
            reverse_code=migrations.RunPython.noop
        )
    ]



import logging

from django.conf import settings
from django.contrib.contenttypes.models import ContentType

from pulpcore.plugin.models import Task
from pulp_ansible.app.models import AnsibleDistribution, AnsibleRepository, CollectionVersion
from pulp_ansible.app.tasks.collections import import_collection

from .promotion import call_copy_task, call_remove_task

log = logging.getLogger(__name__)

GOLDEN_NAME = settings.GALAXY_API_DEFAULT_DISTRIBUTION_BASE_PATH
STAGING_NAME = settings.GALAXY_API_STAGING_DISTRIBUTION_BASE_PATH


def get_created_collection_versions():
    current_task = Task.current()
    created_resources = current_task.created_resources.filter(
        content_type_id=ContentType.objects.get_for_model(CollectionVersion))

    # TODO: replace with values_list
    created_collection_versions = []
    for created_resource in created_resources:
        collection_version = created_resource.content_object

        created_collection_versions.append(collection_version)

    return created_collection_versions


def import_and_move_to_staging(temp_file_pk, **kwargs):
    """Import collection version and move to staging repository.

    Custom task to call pulp_ansible's import_collection() task then
    enqueue two tasks to add to staging repo and remove from inbound repo.

    This task will not wait for the enqueued tasks to finish.
    """
    inbound_repository_pk = kwargs.get('repository_pk')
    import_collection(
        temp_file_pk=temp_file_pk,
        repository_pk=inbound_repository_pk,
        expected_namespace=kwargs['expected_namespace'],
        expected_name=kwargs['expected_name'],
        expected_version=kwargs['expected_version'],
    )

    try:
        staging_repo = AnsibleDistribution.objects.get(name=STAGING_NAME).repository
    except AnsibleRepository.DoesNotExist:
        raise RuntimeError(f'Could not find staging repository: "{STAGING_NAME}"')

    inbound_repo = AnsibleRepository.objects.get(pk=inbound_repository_pk)

    created_collection_versions = get_created_collection_versions()

    for collection_version in created_collection_versions:
        call_copy_task(collection_version, inbound_repo, staging_repo)
        call_remove_task(collection_version, inbound_repo)


def import_and_auto_approve(temp_file_pk, **kwargs):
    """Import collection version and automatically approve.

    Custom task to call pulp_ansible's import_collection() task
    then automatically approve collection version so no
    manual approval action needs to occur.
    """
    inbound_repository_pk = kwargs.get('repository_pk')
    import_collection(
        temp_file_pk=temp_file_pk,
        repository_pk=inbound_repository_pk,
        expected_namespace=kwargs['expected_namespace'],
        expected_name=kwargs['expected_name'],
        expected_version=kwargs['expected_version'],
    )

    try:
        golden_repo = AnsibleDistribution.objects.get(name=GOLDEN_NAME).repository
    except AnsibleRepository.DoesNotExist:
        raise RuntimeError(f'Could not find staging repository: "{GOLDEN_NAME}"')

    inbound_repo = AnsibleRepository.objects.get(pk=inbound_repository_pk)

    created_collection_versions = get_created_collection_versions()

    for collection_version in created_collection_versions:
        call_copy_task(collection_version, inbound_repo, golden_repo)
        call_remove_task(collection_version, inbound_repo)

        log.info('Imported and auto approved collection artifact %s to repository %s',
                 collection_version.relative_path,
                 golden_repo.latest_version())



from django.db import transaction
from django_filters import filters
from django_filters.rest_framework import filterset, DjangoFilterBackend

from pulpcore.app import viewsets
from galaxy_ng.app.access_control import access_policy

from galaxy_ng.app.api.base import LocalSettingsMixin
from galaxy_ng.app.exceptions import ConflictError
from django.contrib.auth.models import Group


class GroupFilter(filterset.FilterSet):
    sort = filters.OrderingFilter(
        fields=(
            ('name', 'name'),
        )
    )

    class Meta:
        model = Group
        fields = {
            'name': ['exact', 'contains', 'startswith']
        }


class GroupViewSet(LocalSettingsMixin, viewsets.GroupViewSet):
    filter_backends = (DjangoFilterBackend,)
    filterset_class = GroupFilter
    permission_classes = [access_policy.GroupAccessPolicy]
    queryset = Group.objects.all()

    # TODO(awcrosby): replace this by setting attribute to None
    # after https://pulp.plan.io/issues/8438 is resolved
    def _remove_attr(self):
        raise AttributeError
    queryset_filtering_required_permission = property(_remove_attr)

    @transaction.atomic
    def create(self, request, *args, **kwargs):
        name = request.data['name']
        if Group.objects.filter(name=name).exists():
            raise ConflictError(
                detail={'name': f'A group named {name} already exists.'}
            )
        return super().create(request, *args, **kwargs)


class GroupModelPermissionViewSet(LocalSettingsMixin, viewsets.GroupModelPermissionViewSet):
    permission_classes = [access_policy.GroupAccessPolicy]


class GroupUserViewSet(LocalSettingsMixin, viewsets.GroupUserViewSet):
    permission_classes = [access_policy.GroupAccessPolicy]



# Generated by Django 2.2.13 on 2020-07-13 18:22

from django.db import migrations, models
import django.db.models.deletion


def set_default_upstream_repository(apps, schema_editor):
    AnsibleRepository = apps.get_model('ansible', 'AnsibleRepository')
    SyncList = apps.get_model('galaxy', 'SyncList')

    db_alias = schema_editor.connection.alias

    synclists = SyncList.objects.using(db_alias).all()

    # Nothing to update
    if not synclists:
        return

    try:
        default_repo = AnsibleRepository.objects.using(db_alias).get(name="automation-hub")
    except AnsibleRepository.DoesNotExist:
        # If there are synclists, but no default repo, then let the exception raise
        if synclists:
            raise

    SyncList.objects.using(db_alias).all().update(upstream_repository=default_repo)


class Migration(migrations.Migration):

    dependencies = [
        ('ansible', '0018_fix_collection_relative_path'),
        ('galaxy', '0002_add_synclist_20200330_squashed'),
        ('core', '0032_export_to_chunks'),
    ]

    operations = [
        migrations.AddField(
            model_name='synclist',
            name='upstream_repository',
            field=models.ForeignKey(to='ansible.AnsibleRepository',
                                    on_delete=django.db.models.deletion.CASCADE,
                                    related_name='upstream_repositories',
                                    null=True),
            ),
        migrations.RunPython(code=set_default_upstream_repository),
        migrations.AlterField(
            model_name='synclist',
            name='upstream_repository',
            field=models.ForeignKey(to='ansible.AnsibleRepository',
                                    on_delete=django.db.models.deletion.CASCADE,
                                    related_name='upstream_repositories',
                                    null=False),
            ),
        migrations.AlterField(
            model_name='synclist',
            name='repository',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='repositories', to='ansible.AnsibleRepository'),
        ),
    ]



from unittest.case import skip
from django.urls.base import reverse
from galaxy_ng.app.constants import DeploymentMode
from django.test.utils import override_settings

from pulp_ansible.app.models import (
    AnsibleDistribution,
    AnsibleRepository,
    Collection,
    CollectionVersion,
)
from galaxy_ng.app import models
from .base import BaseTestCase


def _create_repo(name, **kwargs):
    repo = AnsibleRepository.objects.create(name=name, **kwargs)
    AnsibleDistribution.objects.create(
        name=name, base_path=name, repository=repo
    )
    return repo


def _get_create_version_in_repo(namespace, collection, version, repo):
    collection_version, _ = CollectionVersion.objects.get_or_create(
        namespace=namespace,
        name=collection.name,
        collection=collection,
        version=version,
    )
    qs = CollectionVersion.objects.filter(pk=collection_version.pk)
    with repo.new_version() as new_version:
        new_version.add_content(qs)


@override_settings(GALAXY_DEPLOYMENT_MODE=DeploymentMode.STANDALONE.value)
class TestCollectionViewsets(BaseTestCase):

    def setUp(self):
        super().setUp()

        self.admin_user = self._create_user("admin")
        self.pe_group = self._create_partner_engineer_group()
        self.admin_user.groups.add(self.pe_group)
        self.admin_user.save()

        self.namespace = models.Namespace.objects.create(name='col_namespace')
        self.collection = Collection.objects.create(
            namespace=self.namespace, name='col_collection'
        )
        self.repo = _create_repo(name='col_repo')

        _get_create_version_in_repo(
            self.namespace,
            self.collection,
            '1.1.1',
            self.repo
        )
        _get_create_version_in_repo(
            self.namespace,
            self.collection,
            '1.1.2',
            self.repo
        )

        # TODO: Upload pulp_ansible/tests/assets collection
        #       or create dummy ContentArtifacts directly

        self.collections_url = reverse(
            'galaxy:api:content:v3:collections-list',
            kwargs={
                'path': self.repo.name,
            }
        )

        self.collections_detail_url = reverse(
            'galaxy:api:content:v3:collections-detail',
            kwargs={
                'path': self.repo.name,
                'namespace': self.namespace.name,
                'name': self.collection.name
            }
        )

        self.versions_url = reverse(
            'galaxy:api:content:v3:collection-versions-list',
            kwargs={
                'path': self.repo.name,
                'namespace': self.namespace.name,
                'name': self.collection.name
            }
        )

        self.versions_detail_url = reverse(
            'galaxy:api:content:v3:collection-versions-detail',
            kwargs={
                'path': self.repo.name,
                'namespace': self.namespace.name,
                'name': self.collection.name,
                'version': '1.1.2'
            }
        )

        # The following URLS are temporary deatived due to https://issues.redhat.com/browse/AAH-224
        # metadata urls
        # self.all_collections_url = reverse(
        #     'galaxy:api:content:v3:all-collections-list',
        #     kwargs={
        #         'path': self.repo.name,
        #     }
        # )

        # self.all_versions_url = reverse(
        #     'galaxy:api:content:v3:all-collection-versions-list',
        #     kwargs={
        #         'path': self.repo.name,
        #     }
        # )

        # self.metadata_url = reverse(
        #     'galaxy:api:content:v3:repo-metadata',
        #     kwargs={
        #         'path': self.repo.name,
        #     }
        # )

        # used for href tests
        self.pulp_href_fragment = "pulp_ansible/galaxy"

    def test_collections_list(self):
        """Assert the call to v3/collections returns correct
        collections and versions
        """
        self.client.force_authenticate(user=self.admin_user)
        response = self.client.get(self.collections_url)
        self.assertEqual(response.data['meta']['count'], 1)
        self.assertEqual(response.data['data'][0]['deprecated'], False)
        self.assertEqual(
            response.data['data'][0]['highest_version']['version'], '1.1.2'
        )

        # Ensure href is overwritten
        self.assertNotIn(self.pulp_href_fragment, response.data["data"][0]["href"])
        self.assertNotIn(self.pulp_href_fragment, response.data["data"][0]["versions_url"])
        self.assertNotIn(
            self.pulp_href_fragment, response.data["data"][0]["highest_version"]["href"]
        )

    @skip("https://issues.redhat.com/browse/AAH-224")
    def test_unpaginated_collections_list(self):
        """Assert the call to v3/collections/all returns correct
        collections and versions
        """
        self.client.force_authenticate(user=self.admin_user)
        response = self.client.get(self.all_collections_url)
        self.assertEqual(response.data[0]['deprecated'], False)
        self.assertEqual(response.data[0]['highest_version']['version'], '1.1.2')

        # Ensure href is overwritten
        self.assertNotIn(self.pulp_href_fragment, response.data[0]["href"])
        self.assertNotIn(self.pulp_href_fragment, response.data[0]["versions_url"])
        self.assertNotIn(self.pulp_href_fragment, response.data[0]["highest_version"]["href"])

    def test_collections_detail(self):
        """Assert detail view for v3/collections/namespace/name
        retrurns the same values
        """
        self.client.force_authenticate(user=self.admin_user)
        response = self.client.get(self.collections_detail_url)
        self.assertEqual(response.data['namespace'], self.namespace.name)
        self.assertEqual(response.data['deprecated'], False)
        self.assertEqual(
            response.data['highest_version']['version'], '1.1.2'
        )

        # Ensure hrefs are overwritten
        self.assertNotIn(self.pulp_href_fragment, response.data["href"])
        self.assertNotIn(self.pulp_href_fragment, response.data["versions_url"])
        self.assertNotIn(self.pulp_href_fragment, response.data["highest_version"]["href"])

    def test_collection_versions_list(self):
        """Assert v3/collections/namespace/name/versions/
        lists all available versions.

        Assert that each version href returns correct data and
        artifacts.
        """
        self.client.force_authenticate(user=self.admin_user)
        response = self.client.get(self.versions_url)
        self.assertEqual(response.data['meta']['count'], 2)
        self.assertEqual(response.data['data'][0]['version'], '1.1.2')
        self.assertEqual(response.data['data'][1]['version'], '1.1.1')

        # Ensure hrefs are overwritten
        self.assertNotIn(self.pulp_href_fragment, response.data["data"][0]["href"])

        # TODO: implement subtests for each version after the
        # upload of artifacts has been implemented in `self.setUp`
        # for version in response.data['data']:
        #     with self.subTest(version=version['version):
        #         vresponse = self.client.get(version['href'])
        #         self.assertNotIn(self.pulp_href_fragment, vresponse.data["href"])
        #         self.assertNotIn(self.pulp_href_fragment, vresponse.data["collection"]["href"])
        #         self.assertNotIn(self.pulp_href_fragment, vresponse.data["download_url"])

    # def test_unpaginated_collection_versions_list(self):
    #     """Assert the call to v3/collections/all returns correct
    #     collections and versions
    #     """
    #     self.client.force_authenticate(user=self.admin_user)
    #     response = self.client.get(self.all_versions_url)
    #     self.assertEqual(response.data[0]['version'], '1.1.2')

    #     # Ensure href is overwritten
    #     self.assertNotIn(self.pulp_href_fragment, response.data[0]["href"])
    #     self.assertNotIn(self.pulp_href_fragment, response.data[0]["collection"]["href"])
    #     self.assertNotIn(self.pulp_href_fragment, response.data[0]["download_url"])

    #     # Ensure some fields are in
    #     for field in ('metadata', 'namespace', 'name', 'artifact'):
    #         with self.subTest(field=field):
    #             self.assertIn(field, response.data[0])

    #     # Ensure some fields are not in
    #     for field in ('manifest', 'files'):
    #         with self.subTest(field=field):
    #             self.assertNotIn(field, response.data[0])



from .auth import (
    LoginView,
    LogoutView,
)
from .feature_flags import FeatureFlagsView

__all__ = (
    # auth
    "LoginView",
    "LogoutView",

    # feature_flags
    "FeatureFlagsView",
)



import logging

from django.contrib.auth import models as auth_models

log = logging.getLogger(__name__)

__all__ = (
    "SYSTEM_SCOPE",
    "RH_PARTNER_ENGINEER_GROUP",
    "User",
    "Group",
)


SYSTEM_SCOPE = "system"
RH_PARTNER_ENGINEER_GROUP = f"{SYSTEM_SCOPE}:partner-engineers"


class User(auth_models.AbstractUser):
    """Custom user model."""

    pass


class GroupManager(auth_models.GroupManager):
    def create_identity(self, scope, name):
        return super().create(name=self._make_name(scope, name))

    def get_or_create_identity(self, scope, name):
        group, _ = super().get_or_create(name=self._make_name(scope, name))

        return group, _

    @staticmethod
    def _make_name(scope, name):
        return f"{scope}:{name}"


class Group(auth_models.Group):
    objects = GroupManager()

    class Meta:
        proxy = True

    def account_number(self):
        scope = "rh-identity-account"
        if self.name.startswith(scope):
            account = self.name.replace(f"{scope}:", "", 1)
            return account

        # If not a rh-identity-scoped return full group name
        return self.name



# These access policies are loaded via a post migration signal. To reload them after
# an edit, just rerun the migrations and any changes will get applied.

PULP_CONTAINER_VIEWSETS = {
    # Note. This is the default Pulp Continer access policy with some modifications.
    # Our changes have been marked with comments.
    "distributions/container/container": {
        "statements": [
            {
                "action": ["list"],
                "principal": "authenticated",
                "effect": "allow",
            },
            {
                "action": ["catalog"],
                "principal": "authenticated",
                "effect": "allow",
            },
            {
                "action": ["create"],
                "principal": "authenticated",
                "effect": "allow",
                "condition": "has_model_perms:container.add_containerdistribution",
            },
            {
                "action": ["retrieve"],
                "principal": "authenticated",
                "effect": "allow",
                "condition": [
                    "has_namespace_or_obj_perms:container.view_containerdistribution",
                ],
            },
            {
                "action": ["pull"],
                # Require authentication for container pull.
                # "principal": "*",
                "principal": "authenticated",
                "effect": "allow",
                "condition": [
                    "not is_private",
                ],
            },
            {
                "action": ["pull"],
                "principal": "authenticated",
                "effect": "allow",
                "condition": [
                    "has_namespace_or_obj_perms:container.pull_containerdistribution",
                ],
            },
            {
                "action": ["update", "partial_update"],
                "principal": "authenticated",
                "effect": "allow",
                "condition": [
                    "has_namespace_or_obj_perms:container.change_containerdistribution",
                ],
            },
            {
                "action": ["push"],
                "principal": "authenticated",
                "effect": "allow",
                "condition": [
                    "has_namespace_or_obj_perms:container.push_containerdistribution",
                    "obj_exists",
                ],
            },
            {
                "action": ["push"],
                "principal": "authenticated",
                "effect": "allow",
                "condition": [
                    "has_namespace_or_obj_perms:container.add_containerdistribution",
                    "has_namespace_or_obj_perms:container.push_containerdistribution",
                ],
            },
            {
                "action": ["destroy"],
                "principal": "authenticated",
                "effect": "allow",
                "condition": [
                    "has_namespace_or_obj_perms:container.delete_containerdistribution",
                    "has_namespace_or_obj_perms:container.view_containerdistribution",
                ],
            },
        ],
        # Removed permission assignement. Filtering out the container groups
        # proved to be too much of a challenge.
        "permissions_assignment": []
    },

    "pulp_container/namespaces": {
        "statements": [
            {
                "action": ["list"],
                "principal": "authenticated",
                "effect": "allow",
            },
            {
                "action": ["create"],
                "principal": "authenticated",
                "effect": "allow",
                "condition": "has_model_perms:container.add_containernamespace",
            },
            {
                "action": ["retrieve"],
                "principal": "authenticated",
                "effect": "allow",
                "condition": "has_model_or_obj_perms:container.view_containernamespace",
            },
            {
                "action": ["destroy"],
                "principal": "authenticated",
                "effect": "allow",
                "condition": [
                    "has_model_or_obj_perms:container.delete_containernamespace",
                    "has_model_or_obj_perms:container.view_containernamespace",
                ],
            },
            {
                "action": ["create_distribution"],
                "principal": "authenticated",
                "effect": "allow",
                "condition": "has_model_or_obj_perms:container.namespace_add_containerdistribution",
            },
            {
                "action": ["view_distribution"],
                "principal": "authenticated",
                "effect": "allow",
                "condition": "has_model_or_obj_perms:container.namespace_view_containerdistribution",  # noqa: E501
            },
        ],
        # Removed group creation for owner. Filtering out the container groups proved to be too
        # much of a challenge.
        "permissions_assignment": [
            {
                "function": "add_for_object_creator",
                "parameters": None,
                "permissions": [
                    "container.view_containernamespace",
                    "container.delete_containernamespace",
                    # Add `container.change_containernamespace` permissions so the namespace
                    # owner can add additional groups to their namespace.
                    "container.change_containernamespace",
                    "container.namespace_add_containerdistribution",
                    "container.namespace_delete_containerdistribution",
                    "container.namespace_view_containerdistribution",
                    "container.namespace_pull_containerdistribution",
                    "container.namespace_push_containerdistribution",
                    "container.namespace_change_containerdistribution",
                    "container.namespace_view_containerpushrepository",
                    "container.namespace_modify_content_containerpushrepository",
                ],
            },
        ],
    },


    "repositories/container/container-push": {
        "statements": [
            {
                "action": ["list"],
                "principal": "authenticated",
                "effect": "allow",
            },
            {
                "action": ["retrieve"],
                "principal": "authenticated",
                "effect": "allow",
                "condition": "has_namespace_or_obj_perms:container.view_containerpushrepository",
            },
            {
                "action": ["tag", "untag", "remove_image"],
                "principal": "authenticated",
                "effect": "allow",
                "condition": [
                    "has_namespace_or_obj_perms:container.modify_content_containerpushrepository",
                ],
            },
        ],
        # Remove permission assignment since it's trying to add permissions to groups
        # that don't exist
        "permissions_assignment": [],
    }

}



# Generated by Django 2.2.19 on 2021-03-18 00:37

from django.db import migrations, models
import django.db.models.deletion
import django_lifecycle.mixins
import galaxy_ng.app.access_control.mixins


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0061_call_handle_artifact_checksums_command'),
        ('container', '0018_containerdistribution_description'),
        ('galaxy', '0015_containerdistroreadme'),
    ]

    operations = [
        migrations.CreateModel(
            name='ContainerRegistryRemote',
            fields=[
                ('remote_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='galaxy_containerregistryremote', serialize=False, to='core.Remote')),
            ],
            options={
                'default_related_name': '%(app_label)s_%(model_name)s',
            },
            bases=('core.remote', galaxy_ng.app.access_control.mixins.GroupModelPermissionsMixin),
        ),
        migrations.CreateModel(
            name='ContainerSyncTask',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('repository', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='container.ContainerRepository')),
                ('task', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='core.Task')),
            ],
            options={
                'ordering': ['-task__finished_at'],
            },
        ),
        migrations.CreateModel(
            name='ContainerRegistryRepos',
            fields=[
                ('repository_remote', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, primary_key=True, serialize=False, to='container.ContainerRemote')),
                ('registry', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='galaxy.ContainerRegistryRemote')),
            ],
        ),
    ]



from pulpcore.plugin import PulpPluginAppConfig
from django.db.models.signals import post_migrate
from galaxy_ng.app.access_control.statements import PULP_CONTAINER_VIEWSETS


class PulpGalaxyPluginAppConfig(PulpPluginAppConfig):
    """Entry point for the galaxy plugin."""

    name = "galaxy_ng.app"
    label = "galaxy"
    version = "4.3.3"

    def ready(self):
        super().ready()
        post_migrate.connect(
            set_pulp_container_access_policies,
            sender=self,
            dispatch_uid="override_pulp_container_access_policies"
        )


def set_pulp_container_access_policies(sender, **kwargs):
    apps = kwargs.get("apps")
    if apps is None:
        from django.apps import apps
    AccessPolicy = apps.get_model("core", "AccessPolicy")

    print("Overriding pulp_container access poliicy")
    for view in PULP_CONTAINER_VIEWSETS:
        policy, created = AccessPolicy.objects.update_or_create(
            viewset_name=view, defaults={**PULP_CONTAINER_VIEWSETS[view], "customized": True})



from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response

from galaxy_ng.app.api import base as api_base
from galaxy_ng.app import constants


class APIRootView(api_base.ViewSet):
    permission_classes = [IsAuthenticated]

    def list(self, request, *args, **kwargs):
        return Response({
            'current_version': constants.CURRENT_UI_API_VERSION,
            'available_versions': constants.ALL_UI_API_VERSION
        })



from django.db import migrations

def set_rate_limit_default(apps, schema_editor):
    CollectionRemote = apps.get_model('ansible', 'CollectionRemote')

    # By default pulp core sets the new rate_limit field to null, which remotes the
    # rate limit. This breaks cloud.redhat.com during collection syncs. This sets the
    # default to 8, which should be low enough to not break all of our customer's
    # collection syncs.
    CollectionRemote.objects.all().update(rate_limit=8)


class Migration(migrations.Migration):

    dependencies = [
        ('galaxy', '0017_populate_repos_and_remotes'),
    ]

    operations = [
        migrations.RunPython(
            code=set_rate_limit_default,
            reverse_code=migrations.RunPython.noop
        )
    ]



from django.urls import include, path

from rest_framework import routers

from . import views
from . import viewsets

router = routers.SimpleRouter()
router.register('namespaces', viewsets.NamespaceViewSet, basename='namespaces')

namespace_urls = [
    path("", include(router.urls)),
]

auth_urls = [
    path("auth/token/", views.TokenView.as_view(), name="auth-token"),
]

# these are included in the base router so that they only appear under `content/<distro>/v3`
sync_urls = [
    path(
        "sync/",
        views.SyncRemoteView.as_view(),
        name='sync'
    ),
    path(
        "sync/config/",
        viewsets.SyncConfigViewSet.as_view({'get': 'retrieve', 'put': 'update'}),
        name='sync-config'
    )
]

urlpatterns = [

    # The following endpoints are related to issue https://issues.redhat.com/browse/AAH-224
    # For now endpoints are temporary deactivated until the issue is resolved.
    #
    # path("", viewsets.RepoMetadataViewSet.as_view({"get": "retrieve"}), name="repo-metadata"),
    # path(
    #     'collections/all/',
    #     viewsets.UnpaginatedCollectionViewSet.as_view({'get': 'list'}),
    #     name='all-collections-list'
    # ),
    # path(
    #     'collection_versions/all/',
    #     viewsets.UnpaginatedCollectionVersionViewSet.as_view({"get": "list"}),
    #     name="all-collection-versions-list"
    # ),

    path(
        'collections/',
        viewsets.CollectionViewSet.as_view({'get': 'list'}),
        name='collections-list'
    ),
    path(
        'collections/<str:namespace>/<str:name>/',
        viewsets.CollectionViewSet.as_view({'get': 'retrieve', 'patch': 'update'}),
        name='collections-detail'
    ),
    path(
        'collections/<str:namespace>/<str:name>/versions/',
        viewsets.CollectionVersionViewSet.as_view({'get': 'list'}),
        name='collection-versions-list',
    ),
    path(
        'collections/<str:namespace>/<str:name>/versions/<str:version>/',
        viewsets.CollectionVersionViewSet.as_view({'get': 'retrieve'}),
        name='collection-versions-detail',
    ),
    path(
        'collections/<str:namespace>/<str:name>/versions/<str:version>/docs-blob/',
        viewsets.CollectionVersionDocsViewSet.as_view({'get': 'retrieve'}),
        name='collection-versions-detail-docs',
    ),
    path(
        'imports/collections/<str:pk>/',
        viewsets.CollectionImportViewSet.as_view({'get': 'retrieve'}),
        name='collection-import',
    ),
    path(
        'artifacts/collections/',
        viewsets.CollectionUploadViewSet.as_view({'post': 'create'}),
        name='collection-artifact-upload'
    ),
    path(
        'artifacts/collections/<str:path>/<str:filename>',
        viewsets.CollectionArtifactDownloadView.as_view(),
        name='collection-artifact-download'
    ),
    path(
        'collections/<str:namespace>/<str:name>/versions/<str:version>/move/'
        '<str:source_path>/<str:dest_path>/',
        viewsets.CollectionVersionMoveViewSet.as_view({'post': 'move_content'}),
        name='collection-version-move',
    ),
    path(
        'tasks/',
        viewsets.TaskViewSet.as_view({'get': 'list'}),
        name='tasks-list'
    ),
    path(
        'tasks/<str:pk>/',
        viewsets.TaskViewSet.as_view({'get': 'retrieve'}),
        name='tasks-detail'
    ),
]



import logging
import os
import tempfile
from unittest import mock

from django.conf import settings
from django.test import TestCase, override_settings

from pulpcore.plugin.models import Artifact, PulpTemporaryFile, ContentArtifact
from pulp_ansible.app.models import (
    Collection, CollectionVersion, AnsibleRepository, AnsibleDistribution
)
from pulp_ansible.app.tasks.copy import copy_content

from galaxy_ng.app.tasks import import_and_auto_approve, import_and_move_to_staging
from galaxy_ng.app.tasks.promotion import _remove_content_from_repository


log = logging.getLogger(__name__)
logging.getLogger().setLevel(logging.DEBUG)

golden_name = settings.GALAXY_API_DEFAULT_DISTRIBUTION_BASE_PATH
staging_name = settings.GALAXY_API_STAGING_DISTRIBUTION_BASE_PATH


@override_settings(WORKING_DIRECTORY=tempfile.mkdtemp(suffix='galaxy_ng_unittest'))
class TestTaskPublish(TestCase):
    # artifact_path = os.path.join(tempfile.gettempdir(), 'artifact-tmp')
    artifact_path = os.path.join(settings.WORKING_DIRECTORY, 'artifact-tmp')

    def setUp(self):
        with open(self.artifact_path, 'w') as f:
            f.write('Temp Artifact File')
        self.pulp_temp_file = PulpTemporaryFile.init_and_validate(self.artifact_path)
        self.pulp_temp_file.save()

        self.artifact = Artifact.from_pulp_temporary_file(self.pulp_temp_file)

        collection = Collection.objects.create(namespace='my_ns', name='my_name')
        self.collection_version = CollectionVersion.objects.create(collection=collection)
        self.collection_version.save()

        content_artifact = ContentArtifact.objects.create(
            artifact=self.artifact,
            content=self.collection_version,
        )
        content_artifact.save()

    def test_task_copy_content(self):
        repo1 = AnsibleRepository.objects.get(name=staging_name)
        repo1_version_number = repo1.latest_version().number
        repo2 = AnsibleRepository.objects.get(name='rejected')
        repo2_version_number = repo2.latest_version().number

        self.assertNotIn(
            self.collection_version,
            CollectionVersion.objects.filter(pk__in=repo1.latest_version().content))
        self.assertNotIn(
            self.collection_version,
            CollectionVersion.objects.filter(pk__in=repo2.latest_version().content))

        qs = CollectionVersion.objects.filter(pk=self.collection_version.pk)
        with repo1.new_version() as new_version:
            new_version.add_content(qs)

        self.assertEqual(repo1_version_number + 1, repo1.latest_version().number)
        self.assertIn(
            self.collection_version,
            CollectionVersion.objects.filter(pk__in=repo1.latest_version().content))

        config = [{
            'source_repo_version': repo1.latest_version().pk,
            'dest_repo': repo2.pk,
            'content': [self.collection_version.pk],
        }]
        copy_content(config)

        self.assertEqual(repo2_version_number + 1, repo2.latest_version().number)
        self.assertIn(
            self.collection_version,
            CollectionVersion.objects.filter(pk__in=repo2.latest_version().content))

    def test_task_remove_content_from_repository(self):
        repo = AnsibleRepository.objects.get(name=staging_name)
        repo_version_number = repo.latest_version().number

        qs = CollectionVersion.objects.filter(pk=self.collection_version.pk)
        with repo.new_version() as new_version:
            new_version.add_content(qs)

        self.assertEqual(repo_version_number + 1, repo.latest_version().number)
        self.assertIn(
            self.collection_version,
            CollectionVersion.objects.filter(pk__in=repo.latest_version().content))

        _remove_content_from_repository(self.collection_version.pk, repo.pk)

        self.assertEqual(repo_version_number + 2, repo.latest_version().number)
        self.assertNotIn(
            self.collection_version,
            CollectionVersion.objects.filter(pk__in=repo.latest_version().content))

    @mock.patch('galaxy_ng.app.tasks.publishing.get_created_collection_versions')
    @mock.patch('galaxy_ng.app.tasks.publishing.import_collection')
    @mock.patch('galaxy_ng.app.tasks.promotion.enqueue_with_reservation')
    def test_import_and_auto_approve(self, mocked_enqueue, mocked_import, mocked_get_created):
        inbound_repo = AnsibleRepository.objects.get(name=staging_name)

        golden_repo = AnsibleRepository.objects.get(name=golden_name)

        mocked_get_created.return_value = [self.collection_version]

        import_and_auto_approve(
            self.pulp_temp_file.pk,
            repository_pk=inbound_repo.pk,
            expected_namespace='',
            expected_name='',
            expected_version='',
        )

        self.assertTrue(mocked_import.call_count == 1)
        self.assertTrue(mocked_enqueue.call_count == 2)

        # test cannot find golden repo
        golden_repo.name = 'a_different_name_for_golden'
        golden_repo.save()
        mocked_get_created.side_effect = AnsibleDistribution.DoesNotExist
        with self.assertRaises(AnsibleDistribution.DoesNotExist):
            import_and_auto_approve(
                self.artifact.pk,
                repository_pk=inbound_repo.pk,
                expected_namespace='',
                expected_name='',
                expected_version='',
            )

    @mock.patch('galaxy_ng.app.tasks.publishing.get_created_collection_versions')
    @mock.patch('galaxy_ng.app.tasks.publishing.import_collection')
    @mock.patch('galaxy_ng.app.tasks.promotion.enqueue_with_reservation')
    def test_import_and_move_to_staging(self, mocked_enqueue, mocked_import, mocked_get_created):
        staging_repo = AnsibleRepository.objects.get(name=staging_name)

        inbound_name = 'the_incoming_repo'
        inbound_repo = AnsibleRepository.objects.create(name=inbound_name)
        inbound_dist = AnsibleDistribution(name=inbound_name, base_path=inbound_name)
        inbound_dist.repository = inbound_repo
        inbound_dist.save()

        mocked_get_created.return_value = [self.collection_version]

        import_and_move_to_staging(
            self.pulp_temp_file.pk,
            repository_pk=inbound_repo.pk,
            expected_namespace='',
            expected_name='',
            expected_version='',
        )

        self.assertTrue(mocked_import.call_count == 1)
        self.assertTrue(mocked_enqueue.call_count == 2)

        # test cannot find staging repo
        staging_repo.name = 'a_different_name_for_staging'
        staging_repo.save()
        mocked_get_created.side_effect = AnsibleDistribution.DoesNotExist
        with self.assertRaises(AnsibleDistribution.DoesNotExist):
            import_and_move_to_staging(
                self.pulp_temp_file.pk,
                repository_pk=inbound_repo.pk,
                expected_namespace='',
                expected_name='',
                expected_version='',
            )



from django.utils.translation import gettext_lazy as _
from rest_framework import status
from rest_framework.exceptions import ValidationError


class ConflictError(ValidationError):
    status_code = status.HTTP_409_CONFLICT
    default_detail = _('Data conflicts with existing entity.')
    default_code = 'conflict'



# Generated by Django 2.2.13 on 2020-07-14 21:42

from django.db import migrations, models

def update_policy(apps, schema_editor):
    SyncList = apps.get_model('galaxy', 'SyncList')
    db_alias = schema_editor.connection.alias

    synclists = SyncList.objects.using(db_alias).all().filter(policy__in=('whitelist', 'blacklist'))
    remap = {'whitelist': 'include',
             'blacklist': 'exclude'}

    for synclist in synclists:
        synclist.policy = remap[synclist.policy]
        synclist.save()


class Migration(migrations.Migration):

    dependencies = [
        ('galaxy', '0003_synclist_upstream_and_migrate'),
    ]

    operations = [
        migrations.AlterField(
            model_name='synclist',
            name='policy',
            field=models.CharField(choices=[('exclude', 'exclude'), ('include', 'include'),
                                            ('blacklist', 'blacklist'), ('whitelist', 'whitelist')],
                                   default='exclude', max_length=64),
        ),

        migrations.RunPython(code=update_policy),

        migrations.AlterField(
            model_name='synclist',
            name='policy',
            field=models.CharField(choices=[('exclude', 'exclude'), ('include', 'include')],
                                   default='exclude', max_length=64),
        ),

    ]



import base64

from django.test import override_settings
from django.urls import reverse

from rest_framework import status as http_code
from rest_framework.authtoken import models as token_models
from rest_framework.response import Response
from rest_framework.test import APIClient, APITestCase

from galaxy_ng.app.constants import DeploymentMode
from galaxy_ng.app.models import auth as auth_models

from .base import get_current_ui_url


@override_settings(GALAXY_DEPLOYMENT_MODE=DeploymentMode.STANDALONE.value)
class TestTokenViewStandalone(APITestCase):
    def setUp(self):
        super().setUp()
        self.token_url = reverse("galaxy:api:v3:auth-token")
        self.me_url = get_current_ui_url("me")

        self.user = auth_models.User.objects.create_user(username="test", password="test-secret")

        self.client = APIClient()
        self.client.force_authenticate(user=self.user)

    def _issue_token(self):
        response: Response = self.client.post(self.token_url)
        self.assertEqual(response.status_code, http_code.HTTP_200_OK)
        self.assertTrue("token" in response.data)

        return response.data["token"]

    def test_issue_token(self):
        response: Response = self.client.post(self.token_url)
        self.assertEqual(response.status_code, http_code.HTTP_200_OK)
        self.assertTrue("token" in response.data)
        self.assertEqual(response.data["token"], self._get_token(self.user).key)

    def test_issue_token_basic_auth(self):
        client = APIClient()
        http_authorization = "Basic {}".format(base64.b64encode(b"test:test-secret").decode())
        client.credentials(HTTP_AUTHORIZATION=http_authorization)
        response: Response = client.post(self.token_url)
        self.assertEqual(response.status_code, http_code.HTTP_200_OK)
        self.assertTrue("token" in response.data)

    def test_refresh_token(self):
        token_1 = self._issue_token()
        self.assertTrue(self._token_exists(self.user, token_1))

        token_2 = self._issue_token()

        self.assertNotEqual(token_1, token_2)

        self.assertFalse(self._token_exists(self.user, token_1))
        self.assertTrue(self._token_exists(self.user, token_2))

    def test_token_auth(self):
        token = token_models.Token.objects.create(user=self.user)

        new_client = APIClient()
        new_client.credentials(HTTP_AUTHORIZATION="Token " + token.key)

        response: Response = new_client.get(self.me_url)
        self.assertEqual(response.status_code, http_code.HTTP_200_OK)

    def test_token_auth_missing_token(self):
        new_client = APIClient()

        response: Response = new_client.get(self.me_url)
        self.assertEqual(response.status_code, http_code.HTTP_403_FORBIDDEN)
        self.assertEqual(
            response.data,
            {
                "errors": [
                    {
                        "code": "not_authenticated",
                        "status": "403",
                        "title": "Authentication credentials were not provided.",
                    }
                ]
            },
        )

    def test_token_auth_invalid_token(self):
        new_client = APIClient()
        new_client.credentials(HTTP_AUTHORIZATION="Token c451947e96372bc215c1a9e9e9d01eca910cd144")

        response: Response = new_client.get(self.me_url)
        self.assertEqual(response.status_code, http_code.HTTP_403_FORBIDDEN)
        self.assertEqual(
            response.data,
            {
                "errors": [
                    {
                        "detail": "Invalid token.",
                        "code": "authentication_failed",
                        "status": "403",
                        "title": "Incorrect authentication credentials.",
                    }
                ]
            },
        )

    def test_revoke_token(self):
        token_models.Token.objects.create(user=self.user)

        response: Response = self.client.delete(self.token_url)
        self.assertEqual(response.status_code, http_code.HTTP_204_NO_CONTENT)

        token_exists = token_models.Token.objects.filter(user=self.user).exists()
        self.assertFalse(token_exists)

    @staticmethod
    def _get_token(user):
        return token_models.Token.objects.get(user=user)

    @staticmethod
    def _token_exists(user, token):
        return token_models.Token.objects.filter(user=user, key=token).exists()



# Generated by Django 2.2.11 on 2020-03-31 14:47

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('galaxy', '0001_initial'),
        ('ansible', '0016_add_extension'),
    ]

    operations = [
        migrations.CreateModel(
            name='SyncList',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=64, unique=True)),
                ('policy', models.CharField(choices=[('blacklist', 'blacklist'), ('whitelist', 'whitelist')], default='blacklist', max_length=64)),
                ('groups', models.ManyToManyField(related_name='synclists', to='galaxy.Group')),
                ('repository', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='ansible.AnsibleRepository')),
                ('collections', models.ManyToManyField(to='ansible.Collection')),
                ('namespaces', models.ManyToManyField(to='galaxy.Namespace')),
                ('users', models.ManyToManyField(related_name='synclists', to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



# Generated by Django 2.2.20 on 2021-04-10 17:49

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('galaxy', '0019_containernamespace'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='collectionsynctask',
            options={'ordering': ['-task__pulp_last_updated']},
        ),
    ]



import logging

from rest_framework import status
from rest_framework.test import APIClient

from galaxy_ng.app.models import auth as auth_models
from galaxy_ng.app.constants import DeploymentMode

from .base import BaseTestCase, get_current_ui_url

log = logging.getLogger(__name__)


class TestUiUserViewSet(BaseTestCase):
    def setUp(self):
        super().setUp()
        self.admin_user = auth_models.User.objects.create(username='admin',
                                                          is_superuser=True)
        self.pe_group = self._create_partner_engineer_group()
        self.admin_user.groups.add(self.pe_group)
        self.admin_user.save()

        self.user_url = get_current_ui_url('users-list')
        self.me_url = get_current_ui_url('me')

    def test_super_user(self):
        with self.settings(GALAXY_DEPLOYMENT_MODE=DeploymentMode.STANDALONE.value):
            user = auth_models.User.objects.create(username='haxor')
            self._create_group('', 'test_group1', users=[user], perms=[
                'galaxy.view_user',
                'galaxy.delete_user',
                'galaxy.add_user',
                'galaxy.change_user',
            ])
            self.client.force_authenticate(user=user)
            new_user_data = {
                'username': 'haxor_test1',
                'password': 'cantTouchThis123',
                'groups': [],
                'is_superuser': True,
            }

            # test that regular user's can't create super users
            response = self.client.post(self.user_url, new_user_data, format='json')
            self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

            # test that admins can create super users
            self.client.force_authenticate(user=self.admin_user)
            response = self.client.post(self.user_url, new_user_data, format='json')
            self.assertEqual(response.status_code, status.HTTP_201_CREATED)

            my_data = {
                'username': user.username,
                'password': 'cantTouchThis123',
                'groups': [],
                'is_superuser': True,
            }

            # Test that user's can't elevate their perms
            self.client.force_authenticate(user=user)
            response = self.client.put(self.me_url, my_data, format='json')
            self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

            put_url = '{}{}/'.format(self.user_url, user.id)
            response = self.client.put(put_url, my_data, format='json')
            self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

            # test that the admin can update another user
            self.client.force_authenticate(user=self.admin_user)
            response = self.client.put(put_url, my_data, format='json')
            self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_user_can_only_create_users_with_their_groups(self):
        user = auth_models.User.objects.create(username='haxor')
        group = self._create_group('', 'test_group1', users=[user], perms=[
            'galaxy.view_user',
            'galaxy.delete_user',
            'galaxy.add_user',
            'galaxy.change_user',
        ])
        self.client.force_authenticate(user=user)

        # Verify the user can't create new users with elevated permissions
        new_user_data = {
            'username': 'haxor_test1',
            'password': 'cantTouchThis123',
            'groups': [{
                'id': self.pe_group.id,
                'name': self.pe_group.name
            }]
        }

        response = self.client.post(self.user_url, new_user_data, format='json')
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(response.data['errors'][0]['source']['parameter'], 'groups')

        # Verify the user can create new users with identical permissions
        new_user_data = {
            'username': 'haxor_test2',
            'password': 'cantTouchThis123',
            'groups': [{
                'id': group.id,
                'name': group.name
            }]
        }

        response = self.client.post(self.user_url, new_user_data, format='json')
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    def test_user_can_create_users_with_right_perms(self):
        user = auth_models.User.objects.create(username='haxor')
        self._create_group('', 'test_group1', users=[user], perms=[
            'galaxy.view_user',
            'galaxy.delete_user',
            'galaxy.add_user',
            'galaxy.change_user',
            'galaxy.change_group'
        ])
        self.client.force_authenticate(user=user)

        new_user_data = {
            'username': 'haxor_test3',
            'password': 'cantTouchThis123',
            'groups': [{
                'id': self.pe_group.id,
                'name': self.pe_group.name
            }]
        }

        response = self.client.post(self.user_url, new_user_data, format='json')
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    def test_user_list(self):
        self.client.force_authenticate(user=self.user)
        log.debug('self.client: %s', self.client)
        log.debug('self.client.__dict__: %s', self.client.__dict__)
        with self.settings(GALAXY_DEPLOYMENT_MODE=DeploymentMode.STANDALONE.value):
            response = self.client.get(self.user_url)
            self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

        self.client.force_authenticate(user=self.admin_user)
        with self.settings(GALAXY_DEPLOYMENT_MODE=DeploymentMode.STANDALONE.value):
            response = self.client.get(self.user_url)
            self.assertEqual(response.status_code, status.HTTP_200_OK)
            data = response.data['data']
            self.assertEqual(len(data), auth_models.User.objects.all().count())

        with self.settings(GALAXY_DEPLOYMENT_MODE=DeploymentMode.INSIGHTS.value):
            response = self.client.get(self.user_url)
            self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_user_get(self):
        url = '{}{}/'.format(self.user_url, self.user.id)

        # Users cannot view themselves on the users/ api
        self.client.force_authenticate(user=self.user)
        with self.settings(GALAXY_DEPLOYMENT_MODE=DeploymentMode.STANDALONE.value):
            response = self.client.get(url)
            self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

        # But not other users
        url = '{}{}/'.format(self.user_url, self.admin_user.id)
        with self.settings(GALAXY_DEPLOYMENT_MODE=DeploymentMode.STANDALONE.value):
            response = self.client.get(url)
            self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

        url = '{}{}/'.format(self.user_url, self.user.id)
        self.client.force_authenticate(user=self.admin_user)
        with self.settings(GALAXY_DEPLOYMENT_MODE=DeploymentMode.STANDALONE.value):
            response = self.client.get(url)
            self.assertEqual(response.status_code, status.HTTP_200_OK)
            data = response.data
            self.assertEqual(data['email'], self.user.email)
            self.assertEqual(data['first_name'], self.user.first_name)
            self.assertEqual(len(data['groups']), self.user.groups.all().count())
            for group in data['groups']:
                self.assertTrue(self.user.groups.exists(id=group['id']))

        with self.settings(GALAXY_DEPLOYMENT_MODE=DeploymentMode.INSIGHTS.value):
            response = self.client.get(url)
            self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def _test_create_or_update(self, method_call, url, new_user_data, crud_status, auth_user):
        self.client.force_authenticate(user=auth_user)
        with self.settings(GALAXY_DEPLOYMENT_MODE=DeploymentMode.INSIGHTS.value):
            response = method_call(self.user_url, new_user_data, format='json')
            self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

        with self.settings(GALAXY_DEPLOYMENT_MODE=DeploymentMode.STANDALONE.value):
            # set user with invalid password
            new_user_data['password'] = '12345678'
            response = method_call(url, new_user_data, format='json')
            self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

            error_messages = set([])
            for err in response.data['errors']:
                error_messages.add(err['code'])

            desired_errors = set(
                ['password_too_short', 'password_too_common', 'password_entirely_numeric'])

            self.assertEqual(error_messages, desired_errors)

            # set valid user
            new_user_data['password'] = 'trekkie4Lyfe1701'
            response = method_call(url, new_user_data, format='json')
            self.assertEqual(response.status_code, crud_status)
            data = response.data
            self.assertEqual(data['email'], new_user_data['email'])
            self.assertEqual(data['first_name'], new_user_data['first_name'])
            self.assertEqual(data['groups'], new_user_data['groups'])
            self.assertFalse(
                self.client.login(username=new_user_data['username'], password='bad'))
            self.assertTrue(
                self.client.login(
                    username=new_user_data['username'], password=new_user_data['password']))

        return response

    def test_user_create(self):
        new_user_data = {
            'username': 'test2',
            'first_name': 'First',
            'last_name': 'Last',
            'email': 'email@email.com',
            'groups': [{
                'id': self.pe_group.id,
                'name': self.pe_group.name
            }]
        }

        self.client.force_authenticate(user=self.user)
        with self.settings(GALAXY_DEPLOYMENT_MODE=DeploymentMode.STANDALONE.value):
            response = self.client.post(self.user_url, new_user_data, format='json')
            self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

        self._test_create_or_update(
            self.client.post,
            self.user_url,
            new_user_data,
            status.HTTP_201_CREATED,
            self.admin_user)

    def test_user_update(self):
        user = auth_models.User.objects.create(username='test2')
        put_url = '{}{}/'.format(self.user_url, user.id)
        user.groups.add(self.pe_group)
        user.save()
        new_user_data = {
            'username': 'test2',
            'first_name': 'First',
            'last_name': 'Last',
            'email': 'email@email.com',
            'groups': [{
                'id': self.pe_group.id,
                'name': self.pe_group.name
            }]
        }

        self.client.force_authenticate(user=self.user)
        with self.settings(GALAXY_DEPLOYMENT_MODE=DeploymentMode.STANDALONE.value):
            response = self.client.put(put_url, new_user_data, format='json')
            self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

        self._test_create_or_update(
            self.client.put, put_url, new_user_data, status.HTTP_200_OK, self.admin_user)

    def test_me_get(self):
        self.client.force_authenticate(user=self.user)
        with self.settings(GALAXY_DEPLOYMENT_MODE=DeploymentMode.INSIGHTS.value):
            response = self.client.get(self.me_url)
            self.assertEqual(response.status_code, status.HTTP_200_OK)
            self.assertEqual(response.data['username'], self.user.username)

        with self.settings(GALAXY_DEPLOYMENT_MODE=DeploymentMode.STANDALONE.value):
            response = self.client.get(self.me_url)
            self.assertEqual(response.status_code, status.HTTP_200_OK)
            self.assertEqual(response.data['username'], self.user.username)

    def test_me_update(self):
        user = auth_models.User.objects.create(username='me_test')
        user.save()
        new_user_data = {
            'username': 'test2',
            'first_name': 'First',
            'last_name': 'Last',
            'email': 'email@email.com',
            'groups': []
        }

        self._test_create_or_update(
            self.client.put, self.me_url, new_user_data, status.HTTP_200_OK, user)

        new_user_data = {
            'username': 'test2',
            'first_name': 'First',
            'last_name': 'Last',
            'email': 'email@email.com',
            'groups': [{
                'id': self.pe_group.id,
                'name': self.pe_group.name
            }]
        }

        url = '{}{}/'.format(self.user_url, user.id)

        response = self.client.put(url, new_user_data, format='json')
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_me_delete(self):
        user = auth_models.User.objects.create(username='delete_me_test')
        user.save()

        group = self._create_group('', 'people_that_can_delete_users',
                                   users=[user],
                                   perms=[
                                       'galaxy.view_user',
                                       'galaxy.delete_user',
                                       'galaxy.add_user',
                                       'galaxy.change_user',
                                       'galaxy.change_group'
                                   ])
        self.client.force_authenticate(user=user)

        new_user_data = {
            'username': 'delete_me_test',
            'first_name': 'Del',
            'last_name': 'Eetmi',
            'email': 'email@email.com',
            'groups': [{
                'id': group.id,
                'name': group.name
            }]
        }

        self._test_create_or_update(
            self.client.put, self.me_url, new_user_data, status.HTTP_200_OK, user)

        url = '{}{}/'.format(self.user_url, user.id)

        client = APIClient(raise_request_exception=True)
        client.force_authenticate(user=user)

        response = client.delete(url, format='json')

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

        # should only be one 403 error for this case
        error = response.data['errors'][0]

        self.assertEqual(error['status'], '403')
        self.assertEqual(error['code'], 'permission_denied')

    def test_superuser_can_not_be_deleted(self):
        superuser = auth_models.User.objects.create(username='super_delete_me_test',
                                                    is_superuser=True)
        superuser.save()

        user = auth_models.User.objects.create(username='user_trying_to_delete_superuser')
        user.save()

        group = self._create_group('', 'people_that_can_delete_users',
                                   users=[user],
                                   perms=[
                                       'galaxy.view_user',
                                       'galaxy.delete_user',
                                       'galaxy.add_user',
                                       'galaxy.change_user',
                                       'galaxy.change_group'
                                   ])

        new_user_data = {
            'username': 'user_trying_to_delete_superuser',
            'first_name': 'Del',
            'last_name': 'Sooperuuser',
            'email': 'email@email.com',
            'groups': [{
                'id': group.id,
                'name': group.name
            }]
        }

        self._test_create_or_update(
            self.client.put, self.me_url, new_user_data, status.HTTP_200_OK, user)

        # try to delete a user that is_super_user
        url = '{}{}/'.format(self.user_url, superuser.id)

        client = APIClient(raise_request_exception=True)
        client.force_authenticate(user=user)

        response = client.delete(url, format='json')

        log.debug('response: %s', response)
        log.debug('response.data: %s', response.data)
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

        # Try to delete 'admin' user
        url = '{}{}/'.format(self.user_url, self.admin_user.id)
        response = client.delete(url, format='json')

        log.debug('response: %s', response)
        log.debug('response.data: %s', response.data)
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)



from django.db import migrations
from django.core import exceptions


# note: using assign_perm from guardian.shortcuts doesn't work in migrations
# https://github.com/django-guardian/django-guardian/issues/281#issuecomment-156264129
# The following function is adapted from https://gist.github.com/xuhcc/67871719116bdc0fee6c
def assign_perm(apps, perm, owner):
    PermissionModel = apps.get_model('auth', 'Permission')
    app_label, codename = perm.split('.', 1)

    perm = PermissionModel.objects.get(
        content_type__app_label=app_label,
        codename=codename)

    owner.permissions.add(perm)


def convert_namespace_groups_to_permissions(apps, schema_editor):
    GroupModel = apps.get_model('galaxy', 'Group')
    pe_perms = [
        # groups
        'galaxy.view_group',
        'galaxy.delete_group',
        'galaxy.add_group',
        'galaxy.change_group',

        # users
        'galaxy.view_user',
        'galaxy.delete_user',
        'galaxy.add_user',
        'galaxy.change_user',

        # collections
        'ansible.modify_ansible_repo_content',

        # namespaces
        'galaxy.add_namespace',
        'galaxy.change_namespace',
        'galaxy.upload_to_namespace',
    ]

    try:
        pe_group = GroupModel.objects.get(name="system:partner-engineers")
        for perm in pe_perms:
            assign_perm(apps, perm, pe_group)
    except exceptions.ObjectDoesNotExist:
        pass


class Migration(migrations.Migration):

    dependencies = [
        ('galaxy', '0012_move_collections_by_certification'),
    ]

    operations = [
        migrations.RunPython(
            code=convert_namespace_groups_to_permissions,
        ),
    ]



from galaxy_ng.app.models.auth import Group

from django.contrib.auth import get_user_model
from django.contrib.auth.models import Permission
from django.core.management import BaseCommand, CommandError
from django.core.exceptions import ObjectDoesNotExist

User = get_user_model()


class Command(BaseCommand):
    """
    Django management command for assigning permissions to groups.

    Example:

    assign-permission admins galaxy.add_namespace galaxy.change_namespace
    """

    help = 'Assign one or more permissions to a group'

    def valid_group(self, group):
        try:
            return Group.objects.get(name=group)
        except ObjectDoesNotExist:
            raise CommandError(
                'Group {} does not exist. Please provide a valid '
                'group name.'.format(group))

    def valid_permission(self, permission):
        try:
            app_label, codename = permission.split('.', 1)
        except ValueError:
            raise CommandError(
                "Invalid permission format for {}. "
                "Expecting 'app_label.codename'.".format(permission)
            )
        try:
            return Permission.objects.get(
                content_type__app_label=app_label,
                codename=codename)
        except ObjectDoesNotExist:
            raise CommandError(
                "Permission {} not found. Please provide a valid "
                "permission in the form 'app_label.codename'".format(
                    permission))

    def add_arguments(self, parser):
        parser.add_argument('group', type=self.valid_group)
        parser.add_argument(
            'permissions',
            nargs='+',
            type=self.valid_permission
        )

    def handle(self, *args, **options):
        group = options['group']
        for perm in options['permissions']:
            group.permissions.add(perm.id)
        self.stdout.write("Assigned requested permission to "
                          "group '{}'".format(group.name))



from unittest import mock

from django.conf import settings
from django.urls import reverse

from rest_framework.test import APIClient, APITestCase

from galaxy_ng.app import models
from galaxy_ng.app.access_control import access_policy
from galaxy_ng.app.models import auth as auth_models
from guardian.shortcuts import assign_perm
from galaxy_ng.app import constants


API_PREFIX = settings.GALAXY_API_PATH_PREFIX.strip("/")


def get_current_ui_url(namespace, **kwargs):
    return reverse('galaxy:api:ui:{version}:{namespace}'.format(
        version=constants.CURRENT_UI_API_VERSION,
        namespace=namespace
    ), **kwargs)


class BaseTestCase(APITestCase):
    def setUp(self):
        self.client = APIClient()

        self.user = self._create_user('test')
        self.client.force_authenticate(user=self.user)

        # Permission mock
        patcher = mock.patch.object(
            access_policy.AccessPolicyBase, "has_rh_entitlements", return_value=True
        )
        patcher.start()
        self.addCleanup(patcher.stop)

    @staticmethod
    def _create_user(username):
        return auth_models.User.objects.create(username=username)

    @staticmethod
    def _create_group(scope, name, users=None, perms=[]):
        group, _ = auth_models.Group.objects.get_or_create_identity(scope, name)
        if isinstance(users, auth_models.User):
            users = [users]
        group.user_set.add(*users)
        for p in perms:
            assign_perm(p, group)
        return group

    @staticmethod
    def _create_namespace(name, groups=None):
        namespace = models.Namespace.objects.create(name=name)
        if isinstance(groups, auth_models.Group):
            groups = [groups]

        groups_to_add = {}
        for group in groups:
            groups_to_add[group] = ['galaxy.upload_to_namespace', 'galaxy.change_namespace']
        namespace.groups = groups_to_add
        return namespace

    @staticmethod
    def _create_partner_engineer_group():
        pe_perms = [
            # namespaces
            'galaxy.add_namespace',
            'galaxy.change_namespace',
            'galaxy.upload_to_namespace',

            # collections
            'ansible.modify_ansible_repo_content',

            # users
            'galaxy.view_user',
            'galaxy.delete_user',
            'galaxy.add_user',
            'galaxy.change_user',

            # groups
            'galaxy.view_group',
            'galaxy.delete_group',
            'galaxy.add_group',
            'galaxy.change_group',

            # synclists
            'galaxy.delete_synclist',
            'galaxy.change_synclist',
            'galaxy.view_synclist',
            'galaxy.add_synclist',

            # sync config
            'ansible.change_collectionremote',
        ]
        pe_group = auth_models.Group.objects.create(
            name='partner-engineers')

        for perm in pe_perms:
            assign_perm(perm, pe_group)

        return pe_group



from django.conf import settings

from .base import BaseTestCase, get_current_ui_url


class TestUiFeatureFlagsView(BaseTestCase):
    def setUp(self):
        super().setUp()
        self.original_setting = settings.GALAXY_FEATURE_FLAGS
        settings.GALAXY_FEATURE_FLAGS = {
            'execution_environments': True,
            'widget_x': False,
        }

    def tearDown(self):
        super().tearDown()
        settings.GALAXY_FEATURE_FLAGS = self.original_setting

    def test_feature_flags_url(self):
        self.feature_flags_url = get_current_ui_url('feature-flags')
        response = self.client.get(self.feature_flags_url)
        self.assertEqual(response.data['execution_environments'], True)
        self.assertEqual(response.data['widget_x'], False)



import contextlib
from django.db import models
from django.db import transaction
from django.db import IntegrityError
from django_lifecycle import LifecycleModel
from pulpcore.plugin.models import AutoDeleteObjPermsMixin
from pulp_ansible.app.models import AnsibleRepository, AnsibleDistribution

from galaxy_ng.app.access_control import mixins
from galaxy_ng.app.constants import INBOUND_REPO_NAME_FORMAT

__all__ = ("Namespace", "NamespaceLink")


def create_inbound_repo(name):
    """Creates inbound repo and inbound distribution for namespace publication."""
    inbound_name = INBOUND_REPO_NAME_FORMAT.format(namespace_name=name)
    with contextlib.suppress(IntegrityError):
        # IntegrityError is suppressed for when the named repo/distro already exists
        # In that cases the error handling is performed on the caller.
        repo = AnsibleRepository.objects.create(name=inbound_name)
        AnsibleDistribution.objects.create(
            name=inbound_name,
            base_path=inbound_name,
            repository=repo
        )


def delete_inbound_repo(name):
    """Deletes inbound repo and distro in case of namespace deletion."""
    inbound_name = INBOUND_REPO_NAME_FORMAT.format(namespace_name=name)
    with contextlib.suppress(AnsibleRepository.DoesNotExist):
        AnsibleRepository.objects.get(name=inbound_name).delete()
    with contextlib.suppress(AnsibleDistribution.DoesNotExist):
        AnsibleDistribution.objects.get(name=inbound_name).delete()


class NamespaceManager(models.Manager):

    def create(self, **kwargs):
        """Override to create inbound repo and distro."""
        create_inbound_repo(kwargs['name'])
        return super().create(**kwargs)

    def bulk_create(self, objs, **kwargs):
        for obj in objs:
            create_inbound_repo(obj.name)
        return super().bulk_create(objs, **kwargs)

    def get_or_create(self, *args, **kwargs):
        ns, created = super().get_or_create(*args, **kwargs)
        if created:
            create_inbound_repo(kwargs['name'])
        return ns, created


class Namespace(LifecycleModel, mixins.GroupModelPermissionsMixin, AutoDeleteObjPermsMixin):
    """
    A model representing Ansible content namespace.

    Fields:
        name: Namespace name. Must be lower case containing only alphanumeric
            characters and underscores.
        company: Optional namespace owner company name.
        email: Optional namespace contact email.
        avatar_url: Optional namespace logo URL.
        description: Namespace brief description.
        resources: Namespace resources page in markdown format.

    Relations:
        owners: Reference to namespace owners.
        links: Back reference to related links.

    """
    # Cutom manager to handle inbound repo and distro

    objects = NamespaceManager()

    # Fields

    name = models.CharField(max_length=64, unique=True, blank=False)
    company = models.CharField(max_length=64, blank=True)
    email = models.CharField(max_length=256, blank=True)
    avatar_url = models.URLField(max_length=256, blank=True)
    description = models.CharField(max_length=256, blank=True)
    resources = models.TextField(blank=True)

    def __str__(self):
        return self.name

    @transaction.atomic
    def set_links(self, links):
        """Replace namespace related links with new ones."""
        self.links.all().delete()
        self.links.bulk_create(
            NamespaceLink(name=link["name"], url=link["url"], namespace=self)
            for link in links
        )

    def delete(self, *args, **kwargs):
        delete_inbound_repo(self.name)
        return super().delete(*args, **kwargs)

    class Meta:
        permissions = (
            ('upload_to_namespace', 'Can upload collections to namespace'),
        )


class NamespaceLink(LifecycleModel):
    """
    A model representing a Namespace link.

    Fields:
        name: Link name (e.g. Homepage, Documentation, etc.).
        url: Link URL.

    Relations:
        namespace: Reference to a parent namespace.
    """

    # Fields
    name = models.CharField(max_length=32)
    url = models.URLField(max_length=256)

    # References
    namespace = models.ForeignKey(
        Namespace, on_delete=models.CASCADE, related_name="links"
    )

    def __str__(self):
        return self.name



"""Tests that perform actions over content unit."""
import unittest

from pulp_smash import utils
from pulp_smash.pulp3.utils import delete_orphans

from galaxy_ng.tests.functional.utils import (
    core_client,
    gen_artifact,
    gen_galaxy_client,
    gen_galaxy_content_attrs,
    monitor_task,
    skip_if,
)
from galaxy_ng.tests.functional.utils import set_up_module as setUpModule  # noqa:F401

from pulpcore.client.pulpcore import ArtifactsApi
from pulpcore.client.galaxy_ng import ApiGalaxyCollectionsApi


# Read the instructions provided below for the steps needed to enable this test (see: FIXME's).
@unittest.skip("FIXME: plugin writer action required")
class ContentUnitTestCase(unittest.TestCase):
    """CRUD content unit.

    This test targets the following issues:

    * `Pulp #2872 <https://pulp.plan.io/issues/2872>`_
    * `Pulp #3445 <https://pulp.plan.io/issues/3445>`_
    * `Pulp Smash #870 <https://github.com/pulp/pulp-smash/issues/870>`_
    """

    @classmethod
    def setUpClass(cls):
        """Create class-wide variable."""
        delete_orphans()
        cls.content_unit = {}
        cls.galaxy_content_api = ApiGalaxyCollectionsApi(gen_galaxy_client())
        cls.artifact = gen_artifact()

    @classmethod
    def tearDownClass(cls):
        """Clean class-wide variable."""
        delete_orphans()

    def test_01_create_content_unit(self):
        """Create content unit."""
        attrs = gen_galaxy_content_attrs(self.artifact)
        response = self.galaxy_content_api.create(**attrs)
        created_resources = monitor_task(response.task)
        content_unit = self.galaxy_content_api.read(created_resources[0])
        self.content_unit.update(content_unit.to_dict())
        for key, val in attrs.items():
            with self.subTest(key=key):
                self.assertEqual(self.content_unit[key], val)

    @skip_if(bool, "content_unit", False)
    def test_02_read_content_unit(self):
        """Read a content unit by its href."""
        content_unit = self.galaxy_content_api.read(self.content_unit["pulp_href"]).to_dict()
        for key, val in self.content_unit.items():
            with self.subTest(key=key):
                self.assertEqual(content_unit[key], val)

    @skip_if(bool, "content_unit", False)
    def test_02_read_content_units(self):
        """Read a content unit by its relative_path."""
        # FIXME: "relative_path" is an attribute specific to the File plugin. It is only an
        # example. You should replace this with some other field specific to your content type.
        page = self.galaxy_content_api.list(relative_path=self.content_unit["relative_path"])
        self.assertEqual(len(page.results), 1)
        for key, val in self.content_unit.items():
            with self.subTest(key=key):
                self.assertEqual(page.results[0].to_dict()[key], val)

    @skip_if(bool, "content_unit", False)
    def test_03_partially_update(self):
        """Attempt to update a content unit using HTTP PATCH.

        This HTTP method is not supported and a HTTP exception is expected.
        """
        attrs = gen_galaxy_content_attrs(self.artifact)
        with self.assertRaises(AttributeError) as exc:
            self.galaxy_content_api.partial_update(self.content_unit["pulp_href"], attrs)
        msg = "object has no attribute 'partial_update'"
        self.assertIn(msg, exc.exception.args[0])

    @skip_if(bool, "content_unit", False)
    def test_03_fully_update(self):
        """Attempt to update a content unit using HTTP PUT.

        This HTTP method is not supported and a HTTP exception is expected.
        """
        attrs = gen_galaxy_content_attrs(self.artifact)
        with self.assertRaises(AttributeError) as exc:
            self.galaxy_content_api.update(self.content_unit["pulp_href"], attrs)
        msg = "object has no attribute 'update'"
        self.assertIn(msg, exc.exception.args[0])

    @skip_if(bool, "content_unit", False)
    def test_04_delete(self):
        """Attempt to delete a content unit using HTTP DELETE.

        This HTTP method is not supported and a HTTP exception is expected.
        """
        with self.assertRaises(AttributeError) as exc:
            self.galaxy_content_api.delete(self.content_unit["pulp_href"])
        msg = "object has no attribute 'delete'"
        self.assertIn(msg, exc.exception.args[0])
